# This file was automatically generated by SWIG (http://www.swig.org).
# Version 4.0.2
#
# Do not make changes to this file unless you know what you are doing--modify
# the SWIG interface file instead.

from sys import version_info as _swig_python_version_info
if _swig_python_version_info < (2, 7, 0):
    raise RuntimeError("Python 2.7 or later required")

# Import the low-level C/C++ module
if __package__ or "." in __name__:
    from . import _ode
else:
    import _ode

try:
    import builtins as __builtin__
except ImportError:
    import __builtin__

_swig_new_instance_method = _ode.SWIG_PyInstanceMethod_New
_swig_new_static_method = _ode.SWIG_PyStaticMethod_New

def _swig_repr(self):
    try:
        strthis = "proxy of " + self.this.__repr__()
    except __builtin__.Exception:
        strthis = ""
    return "<%s.%s; %s >" % (self.__class__.__module__, self.__class__.__name__, strthis,)


def _swig_setattr_nondynamic_instance_variable(set):
    def set_instance_attr(self, name, value):
        if name == "thisown":
            self.this.own(value)
        elif name == "this":
            set(self, name, value)
        elif hasattr(self, name) and isinstance(getattr(type(self), name), property):
            set(self, name, value)
        else:
            raise AttributeError("You cannot add instance attributes to %s" % self)
    return set_instance_attr


def _swig_setattr_nondynamic_class_variable(set):
    def set_class_attr(cls, name, value):
        if hasattr(cls, name) and not isinstance(getattr(cls, name), property):
            set(cls, name, value)
        else:
            raise AttributeError("You cannot add class attributes to %s" % cls)
    return set_class_attr


def _swig_add_metaclass(metaclass):
    """Class decorator for adding a metaclass to a SWIG wrapped class - a slimmed down version of six.add_metaclass"""
    def wrapper(cls):
        return metaclass(cls.__name__, cls.__bases__, cls.__dict__.copy())
    return wrapper


class _SwigNonDynamicMeta(type):
    """Meta class to enforce nondynamic attributes (no new attributes) for a class"""
    __setattr__ = _swig_setattr_nondynamic_class_variable(type.__setattr__)


import weakref

import mfem._par.vector
import mfem._par.array
import mfem._par.mem_manager
import mfem._par.operators
class ODESolver(object):
    r"""Proxy of C++ mfem::ODESolver class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")

    def __init__(self, *args, **kwargs):
        raise AttributeError("No constructor defined - class is abstract")
    __repr__ = _swig_repr

    def Init(self, f):
        r"""Init(ODESolver self, TimeDependentOperator f)"""
        return _ode.ODESolver_Init(self, f)
    Init = _swig_new_instance_method(_ode.ODESolver_Init)

    def Step(self, x, t, dt):
        r"""Step(ODESolver self, Vector x, double & t, double & dt)"""
        return _ode.ODESolver_Step(self, x, t, dt)
    Step = _swig_new_instance_method(_ode.ODESolver_Step)

    def Run(self, x, t, dt, tf):
        r"""Run(ODESolver self, Vector x, double & t, double & dt, double tf)"""
        return _ode.ODESolver_Run(self, x, t, dt, tf)
    Run = _swig_new_instance_method(_ode.ODESolver_Run)

    def GetMaxStateSize(self):
        r"""GetMaxStateSize(ODESolver self) -> int"""
        return _ode.ODESolver_GetMaxStateSize(self)
    GetMaxStateSize = _swig_new_instance_method(_ode.ODESolver_GetMaxStateSize)

    def GetStateSize(self):
        r"""GetStateSize(ODESolver self) -> int"""
        return _ode.ODESolver_GetStateSize(self)
    GetStateSize = _swig_new_instance_method(_ode.ODESolver_GetStateSize)

    def GetStateVector(self, *args):
        r"""
        GetStateVector(ODESolver self, int i) -> Vector
        GetStateVector(ODESolver self, int i, Vector state)
        """
        return _ode.ODESolver_GetStateVector(self, *args)
    GetStateVector = _swig_new_instance_method(_ode.ODESolver_GetStateVector)

    def SetStateVector(self, i, state):
        r"""SetStateVector(ODESolver self, int i, Vector state)"""
        return _ode.ODESolver_SetStateVector(self, i, state)
    SetStateVector = _swig_new_instance_method(_ode.ODESolver_SetStateVector)
    __swig_destroy__ = _ode.delete_ODESolver

# Register ODESolver in _ode:
_ode.ODESolver_swigregister(ODESolver)

class ForwardEulerSolver(ODESolver):
    r"""Proxy of C++ mfem::ForwardEulerSolver class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def Init(self, _f):
        r"""Init(ForwardEulerSolver self, TimeDependentOperator _f)"""
        return _ode.ForwardEulerSolver_Init(self, _f)
    Init = _swig_new_instance_method(_ode.ForwardEulerSolver_Init)

    def Step(self, x, t, dt):
        r"""Step(ForwardEulerSolver self, Vector x, double & t, double & dt)"""
        return _ode.ForwardEulerSolver_Step(self, x, t, dt)
    Step = _swig_new_instance_method(_ode.ForwardEulerSolver_Step)

    def __init__(self):
        r"""__init__(ForwardEulerSolver self) -> ForwardEulerSolver"""
        _ode.ForwardEulerSolver_swiginit(self, _ode.new_ForwardEulerSolver())
    __swig_destroy__ = _ode.delete_ForwardEulerSolver

# Register ForwardEulerSolver in _ode:
_ode.ForwardEulerSolver_swigregister(ForwardEulerSolver)

class RK2Solver(ODESolver):
    r"""Proxy of C++ mfem::RK2Solver class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args, **kwargs):
        r"""__init__(RK2Solver self, double const _a=2./3.) -> RK2Solver"""
        _ode.RK2Solver_swiginit(self, _ode.new_RK2Solver(*args, **kwargs))

    def Init(self, _f):
        r"""Init(RK2Solver self, TimeDependentOperator _f)"""
        return _ode.RK2Solver_Init(self, _f)
    Init = _swig_new_instance_method(_ode.RK2Solver_Init)

    def Step(self, x, t, dt):
        r"""Step(RK2Solver self, Vector x, double & t, double & dt)"""
        return _ode.RK2Solver_Step(self, x, t, dt)
    Step = _swig_new_instance_method(_ode.RK2Solver_Step)
    __swig_destroy__ = _ode.delete_RK2Solver

# Register RK2Solver in _ode:
_ode.RK2Solver_swigregister(RK2Solver)

class RK3SSPSolver(ODESolver):
    r"""Proxy of C++ mfem::RK3SSPSolver class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def Init(self, _f):
        r"""Init(RK3SSPSolver self, TimeDependentOperator _f)"""
        return _ode.RK3SSPSolver_Init(self, _f)
    Init = _swig_new_instance_method(_ode.RK3SSPSolver_Init)

    def Step(self, x, t, dt):
        r"""Step(RK3SSPSolver self, Vector x, double & t, double & dt)"""
        return _ode.RK3SSPSolver_Step(self, x, t, dt)
    Step = _swig_new_instance_method(_ode.RK3SSPSolver_Step)

    def __init__(self):
        r"""__init__(RK3SSPSolver self) -> RK3SSPSolver"""
        _ode.RK3SSPSolver_swiginit(self, _ode.new_RK3SSPSolver())
    __swig_destroy__ = _ode.delete_RK3SSPSolver

# Register RK3SSPSolver in _ode:
_ode.RK3SSPSolver_swigregister(RK3SSPSolver)

class RK4Solver(ODESolver):
    r"""Proxy of C++ mfem::RK4Solver class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def Init(self, _f):
        r"""Init(RK4Solver self, TimeDependentOperator _f)"""
        return _ode.RK4Solver_Init(self, _f)
    Init = _swig_new_instance_method(_ode.RK4Solver_Init)

    def Step(self, x, t, dt):
        r"""Step(RK4Solver self, Vector x, double & t, double & dt)"""
        return _ode.RK4Solver_Step(self, x, t, dt)
    Step = _swig_new_instance_method(_ode.RK4Solver_Step)

    def __init__(self):
        r"""__init__(RK4Solver self) -> RK4Solver"""
        _ode.RK4Solver_swiginit(self, _ode.new_RK4Solver())
    __swig_destroy__ = _ode.delete_RK4Solver

# Register RK4Solver in _ode:
_ode.RK4Solver_swigregister(RK4Solver)

class ExplicitRKSolver(ODESolver):
    r"""Proxy of C++ mfem::ExplicitRKSolver class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, _s, _a, _b, _c):
        r"""__init__(ExplicitRKSolver self, int _s, double const * _a, double const * _b, double const * _c) -> ExplicitRKSolver"""
        _ode.ExplicitRKSolver_swiginit(self, _ode.new_ExplicitRKSolver(_s, _a, _b, _c))

    def Init(self, _f):
        r"""Init(ExplicitRKSolver self, TimeDependentOperator _f)"""
        return _ode.ExplicitRKSolver_Init(self, _f)
    Init = _swig_new_instance_method(_ode.ExplicitRKSolver_Init)

    def Step(self, x, t, dt):
        r"""Step(ExplicitRKSolver self, Vector x, double & t, double & dt)"""
        return _ode.ExplicitRKSolver_Step(self, x, t, dt)
    Step = _swig_new_instance_method(_ode.ExplicitRKSolver_Step)
    __swig_destroy__ = _ode.delete_ExplicitRKSolver

# Register ExplicitRKSolver in _ode:
_ode.ExplicitRKSolver_swigregister(ExplicitRKSolver)

class RK6Solver(ExplicitRKSolver):
    r"""Proxy of C++ mfem::RK6Solver class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self):
        r"""__init__(RK6Solver self) -> RK6Solver"""
        _ode.RK6Solver_swiginit(self, _ode.new_RK6Solver())
    __swig_destroy__ = _ode.delete_RK6Solver

# Register RK6Solver in _ode:
_ode.RK6Solver_swigregister(RK6Solver)

class RK8Solver(ExplicitRKSolver):
    r"""Proxy of C++ mfem::RK8Solver class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self):
        r"""__init__(RK8Solver self) -> RK8Solver"""
        _ode.RK8Solver_swiginit(self, _ode.new_RK8Solver())
    __swig_destroy__ = _ode.delete_RK8Solver

# Register RK8Solver in _ode:
_ode.RK8Solver_swigregister(RK8Solver)

class AdamsBashforthSolver(ODESolver):
    r"""Proxy of C++ mfem::AdamsBashforthSolver class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, _s, _a):
        r"""__init__(AdamsBashforthSolver self, int _s, double const * _a) -> AdamsBashforthSolver"""
        _ode.AdamsBashforthSolver_swiginit(self, _ode.new_AdamsBashforthSolver(_s, _a))

    def Init(self, _f):
        r"""Init(AdamsBashforthSolver self, TimeDependentOperator _f)"""
        return _ode.AdamsBashforthSolver_Init(self, _f)
    Init = _swig_new_instance_method(_ode.AdamsBashforthSolver_Init)

    def Step(self, x, t, dt):
        r"""Step(AdamsBashforthSolver self, Vector x, double & t, double & dt)"""
        return _ode.AdamsBashforthSolver_Step(self, x, t, dt)
    Step = _swig_new_instance_method(_ode.AdamsBashforthSolver_Step)

    def GetMaxStateSize(self):
        r"""GetMaxStateSize(AdamsBashforthSolver self) -> int"""
        return _ode.AdamsBashforthSolver_GetMaxStateSize(self)
    GetMaxStateSize = _swig_new_instance_method(_ode.AdamsBashforthSolver_GetMaxStateSize)

    def GetStateSize(self):
        r"""GetStateSize(AdamsBashforthSolver self) -> int"""
        return _ode.AdamsBashforthSolver_GetStateSize(self)
    GetStateSize = _swig_new_instance_method(_ode.AdamsBashforthSolver_GetStateSize)

    def GetStateVector(self, *args):
        r"""
        GetStateVector(AdamsBashforthSolver self, int i) -> Vector
        GetStateVector(AdamsBashforthSolver self, int i, Vector state)
        """
        return _ode.AdamsBashforthSolver_GetStateVector(self, *args)
    GetStateVector = _swig_new_instance_method(_ode.AdamsBashforthSolver_GetStateVector)

    def SetStateVector(self, i, state):
        r"""SetStateVector(AdamsBashforthSolver self, int i, Vector state)"""
        return _ode.AdamsBashforthSolver_SetStateVector(self, i, state)
    SetStateVector = _swig_new_instance_method(_ode.AdamsBashforthSolver_SetStateVector)
    __swig_destroy__ = _ode.delete_AdamsBashforthSolver

# Register AdamsBashforthSolver in _ode:
_ode.AdamsBashforthSolver_swigregister(AdamsBashforthSolver)

class AB1Solver(AdamsBashforthSolver):
    r"""Proxy of C++ mfem::AB1Solver class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self):
        r"""__init__(AB1Solver self) -> AB1Solver"""
        _ode.AB1Solver_swiginit(self, _ode.new_AB1Solver())
    __swig_destroy__ = _ode.delete_AB1Solver

# Register AB1Solver in _ode:
_ode.AB1Solver_swigregister(AB1Solver)

class AB2Solver(AdamsBashforthSolver):
    r"""Proxy of C++ mfem::AB2Solver class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self):
        r"""__init__(AB2Solver self) -> AB2Solver"""
        _ode.AB2Solver_swiginit(self, _ode.new_AB2Solver())
    __swig_destroy__ = _ode.delete_AB2Solver

# Register AB2Solver in _ode:
_ode.AB2Solver_swigregister(AB2Solver)

class AB3Solver(AdamsBashforthSolver):
    r"""Proxy of C++ mfem::AB3Solver class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self):
        r"""__init__(AB3Solver self) -> AB3Solver"""
        _ode.AB3Solver_swiginit(self, _ode.new_AB3Solver())
    __swig_destroy__ = _ode.delete_AB3Solver

# Register AB3Solver in _ode:
_ode.AB3Solver_swigregister(AB3Solver)

class AB4Solver(AdamsBashforthSolver):
    r"""Proxy of C++ mfem::AB4Solver class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self):
        r"""__init__(AB4Solver self) -> AB4Solver"""
        _ode.AB4Solver_swiginit(self, _ode.new_AB4Solver())
    __swig_destroy__ = _ode.delete_AB4Solver

# Register AB4Solver in _ode:
_ode.AB4Solver_swigregister(AB4Solver)

class AB5Solver(AdamsBashforthSolver):
    r"""Proxy of C++ mfem::AB5Solver class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self):
        r"""__init__(AB5Solver self) -> AB5Solver"""
        _ode.AB5Solver_swiginit(self, _ode.new_AB5Solver())
    __swig_destroy__ = _ode.delete_AB5Solver

# Register AB5Solver in _ode:
_ode.AB5Solver_swigregister(AB5Solver)

class AdamsMoultonSolver(ODESolver):
    r"""Proxy of C++ mfem::AdamsMoultonSolver class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, _s, _a):
        r"""__init__(AdamsMoultonSolver self, int _s, double const * _a) -> AdamsMoultonSolver"""
        _ode.AdamsMoultonSolver_swiginit(self, _ode.new_AdamsMoultonSolver(_s, _a))

    def Init(self, _f):
        r"""Init(AdamsMoultonSolver self, TimeDependentOperator _f)"""
        return _ode.AdamsMoultonSolver_Init(self, _f)
    Init = _swig_new_instance_method(_ode.AdamsMoultonSolver_Init)

    def Step(self, x, t, dt):
        r"""Step(AdamsMoultonSolver self, Vector x, double & t, double & dt)"""
        return _ode.AdamsMoultonSolver_Step(self, x, t, dt)
    Step = _swig_new_instance_method(_ode.AdamsMoultonSolver_Step)

    def GetMaxStateSize(self):
        r"""GetMaxStateSize(AdamsMoultonSolver self) -> int"""
        return _ode.AdamsMoultonSolver_GetMaxStateSize(self)
    GetMaxStateSize = _swig_new_instance_method(_ode.AdamsMoultonSolver_GetMaxStateSize)

    def GetStateSize(self):
        r"""GetStateSize(AdamsMoultonSolver self) -> int"""
        return _ode.AdamsMoultonSolver_GetStateSize(self)
    GetStateSize = _swig_new_instance_method(_ode.AdamsMoultonSolver_GetStateSize)

    def GetStateVector(self, *args):
        r"""
        GetStateVector(AdamsMoultonSolver self, int i) -> Vector
        GetStateVector(AdamsMoultonSolver self, int i, Vector state)
        """
        return _ode.AdamsMoultonSolver_GetStateVector(self, *args)
    GetStateVector = _swig_new_instance_method(_ode.AdamsMoultonSolver_GetStateVector)

    def SetStateVector(self, i, state):
        r"""SetStateVector(AdamsMoultonSolver self, int i, Vector state)"""
        return _ode.AdamsMoultonSolver_SetStateVector(self, i, state)
    SetStateVector = _swig_new_instance_method(_ode.AdamsMoultonSolver_SetStateVector)
    __swig_destroy__ = _ode.delete_AdamsMoultonSolver

# Register AdamsMoultonSolver in _ode:
_ode.AdamsMoultonSolver_swigregister(AdamsMoultonSolver)

class AM0Solver(AdamsMoultonSolver):
    r"""Proxy of C++ mfem::AM0Solver class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self):
        r"""__init__(AM0Solver self) -> AM0Solver"""
        _ode.AM0Solver_swiginit(self, _ode.new_AM0Solver())
    __swig_destroy__ = _ode.delete_AM0Solver

# Register AM0Solver in _ode:
_ode.AM0Solver_swigregister(AM0Solver)

class AM1Solver(AdamsMoultonSolver):
    r"""Proxy of C++ mfem::AM1Solver class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self):
        r"""__init__(AM1Solver self) -> AM1Solver"""
        _ode.AM1Solver_swiginit(self, _ode.new_AM1Solver())
    __swig_destroy__ = _ode.delete_AM1Solver

# Register AM1Solver in _ode:
_ode.AM1Solver_swigregister(AM1Solver)

class AM2Solver(AdamsMoultonSolver):
    r"""Proxy of C++ mfem::AM2Solver class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self):
        r"""__init__(AM2Solver self) -> AM2Solver"""
        _ode.AM2Solver_swiginit(self, _ode.new_AM2Solver())
    __swig_destroy__ = _ode.delete_AM2Solver

# Register AM2Solver in _ode:
_ode.AM2Solver_swigregister(AM2Solver)

class AM3Solver(AdamsMoultonSolver):
    r"""Proxy of C++ mfem::AM3Solver class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self):
        r"""__init__(AM3Solver self) -> AM3Solver"""
        _ode.AM3Solver_swiginit(self, _ode.new_AM3Solver())
    __swig_destroy__ = _ode.delete_AM3Solver

# Register AM3Solver in _ode:
_ode.AM3Solver_swigregister(AM3Solver)

class AM4Solver(AdamsMoultonSolver):
    r"""Proxy of C++ mfem::AM4Solver class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self):
        r"""__init__(AM4Solver self) -> AM4Solver"""
        _ode.AM4Solver_swiginit(self, _ode.new_AM4Solver())
    __swig_destroy__ = _ode.delete_AM4Solver

# Register AM4Solver in _ode:
_ode.AM4Solver_swigregister(AM4Solver)

class BackwardEulerSolver(ODESolver):
    r"""Proxy of C++ mfem::BackwardEulerSolver class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def Init(self, _f):
        r"""Init(BackwardEulerSolver self, TimeDependentOperator _f)"""
        return _ode.BackwardEulerSolver_Init(self, _f)
    Init = _swig_new_instance_method(_ode.BackwardEulerSolver_Init)

    def Step(self, x, t, dt):
        r"""Step(BackwardEulerSolver self, Vector x, double & t, double & dt)"""
        return _ode.BackwardEulerSolver_Step(self, x, t, dt)
    Step = _swig_new_instance_method(_ode.BackwardEulerSolver_Step)

    def __init__(self):
        r"""__init__(BackwardEulerSolver self) -> BackwardEulerSolver"""
        _ode.BackwardEulerSolver_swiginit(self, _ode.new_BackwardEulerSolver())
    __swig_destroy__ = _ode.delete_BackwardEulerSolver

# Register BackwardEulerSolver in _ode:
_ode.BackwardEulerSolver_swigregister(BackwardEulerSolver)

class ImplicitMidpointSolver(ODESolver):
    r"""Proxy of C++ mfem::ImplicitMidpointSolver class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def Init(self, _f):
        r"""Init(ImplicitMidpointSolver self, TimeDependentOperator _f)"""
        return _ode.ImplicitMidpointSolver_Init(self, _f)
    Init = _swig_new_instance_method(_ode.ImplicitMidpointSolver_Init)

    def Step(self, x, t, dt):
        r"""Step(ImplicitMidpointSolver self, Vector x, double & t, double & dt)"""
        return _ode.ImplicitMidpointSolver_Step(self, x, t, dt)
    Step = _swig_new_instance_method(_ode.ImplicitMidpointSolver_Step)

    def __init__(self):
        r"""__init__(ImplicitMidpointSolver self) -> ImplicitMidpointSolver"""
        _ode.ImplicitMidpointSolver_swiginit(self, _ode.new_ImplicitMidpointSolver())
    __swig_destroy__ = _ode.delete_ImplicitMidpointSolver

# Register ImplicitMidpointSolver in _ode:
_ode.ImplicitMidpointSolver_swigregister(ImplicitMidpointSolver)

class SDIRK23Solver(ODESolver):
    r"""Proxy of C++ mfem::SDIRK23Solver class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, gamma_opt=1):
        r"""__init__(SDIRK23Solver self, int gamma_opt=1) -> SDIRK23Solver"""
        _ode.SDIRK23Solver_swiginit(self, _ode.new_SDIRK23Solver(gamma_opt))

    def Init(self, _f):
        r"""Init(SDIRK23Solver self, TimeDependentOperator _f)"""
        return _ode.SDIRK23Solver_Init(self, _f)
    Init = _swig_new_instance_method(_ode.SDIRK23Solver_Init)

    def Step(self, x, t, dt):
        r"""Step(SDIRK23Solver self, Vector x, double & t, double & dt)"""
        return _ode.SDIRK23Solver_Step(self, x, t, dt)
    Step = _swig_new_instance_method(_ode.SDIRK23Solver_Step)
    __swig_destroy__ = _ode.delete_SDIRK23Solver

# Register SDIRK23Solver in _ode:
_ode.SDIRK23Solver_swigregister(SDIRK23Solver)

class SDIRK34Solver(ODESolver):
    r"""Proxy of C++ mfem::SDIRK34Solver class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def Init(self, _f):
        r"""Init(SDIRK34Solver self, TimeDependentOperator _f)"""
        return _ode.SDIRK34Solver_Init(self, _f)
    Init = _swig_new_instance_method(_ode.SDIRK34Solver_Init)

    def Step(self, x, t, dt):
        r"""Step(SDIRK34Solver self, Vector x, double & t, double & dt)"""
        return _ode.SDIRK34Solver_Step(self, x, t, dt)
    Step = _swig_new_instance_method(_ode.SDIRK34Solver_Step)

    def __init__(self):
        r"""__init__(SDIRK34Solver self) -> SDIRK34Solver"""
        _ode.SDIRK34Solver_swiginit(self, _ode.new_SDIRK34Solver())
    __swig_destroy__ = _ode.delete_SDIRK34Solver

# Register SDIRK34Solver in _ode:
_ode.SDIRK34Solver_swigregister(SDIRK34Solver)

class SDIRK33Solver(ODESolver):
    r"""Proxy of C++ mfem::SDIRK33Solver class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def Init(self, _f):
        r"""Init(SDIRK33Solver self, TimeDependentOperator _f)"""
        return _ode.SDIRK33Solver_Init(self, _f)
    Init = _swig_new_instance_method(_ode.SDIRK33Solver_Init)

    def Step(self, x, t, dt):
        r"""Step(SDIRK33Solver self, Vector x, double & t, double & dt)"""
        return _ode.SDIRK33Solver_Step(self, x, t, dt)
    Step = _swig_new_instance_method(_ode.SDIRK33Solver_Step)

    def __init__(self):
        r"""__init__(SDIRK33Solver self) -> SDIRK33Solver"""
        _ode.SDIRK33Solver_swiginit(self, _ode.new_SDIRK33Solver())
    __swig_destroy__ = _ode.delete_SDIRK33Solver

# Register SDIRK33Solver in _ode:
_ode.SDIRK33Solver_swigregister(SDIRK33Solver)

class GeneralizedAlphaSolver(ODESolver):
    r"""Proxy of C++ mfem::GeneralizedAlphaSolver class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, rho=1.0):
        r"""__init__(GeneralizedAlphaSolver self, double rho=1.0) -> GeneralizedAlphaSolver"""
        _ode.GeneralizedAlphaSolver_swiginit(self, _ode.new_GeneralizedAlphaSolver(rho))

    def Init(self, _f):
        r"""Init(GeneralizedAlphaSolver self, TimeDependentOperator _f)"""
        return _ode.GeneralizedAlphaSolver_Init(self, _f)
    Init = _swig_new_instance_method(_ode.GeneralizedAlphaSolver_Init)

    def Step(self, x, t, dt):
        r"""Step(GeneralizedAlphaSolver self, Vector x, double & t, double & dt)"""
        return _ode.GeneralizedAlphaSolver_Step(self, x, t, dt)
    Step = _swig_new_instance_method(_ode.GeneralizedAlphaSolver_Step)

    def GetMaxStateSize(self):
        r"""GetMaxStateSize(GeneralizedAlphaSolver self) -> int"""
        return _ode.GeneralizedAlphaSolver_GetMaxStateSize(self)
    GetMaxStateSize = _swig_new_instance_method(_ode.GeneralizedAlphaSolver_GetMaxStateSize)

    def GetStateSize(self):
        r"""GetStateSize(GeneralizedAlphaSolver self) -> int"""
        return _ode.GeneralizedAlphaSolver_GetStateSize(self)
    GetStateSize = _swig_new_instance_method(_ode.GeneralizedAlphaSolver_GetStateSize)

    def GetStateVector(self, *args):
        r"""
        GetStateVector(GeneralizedAlphaSolver self, int i) -> Vector
        GetStateVector(GeneralizedAlphaSolver self, int i, Vector state)
        """
        return _ode.GeneralizedAlphaSolver_GetStateVector(self, *args)
    GetStateVector = _swig_new_instance_method(_ode.GeneralizedAlphaSolver_GetStateVector)

    def SetStateVector(self, i, state):
        r"""SetStateVector(GeneralizedAlphaSolver self, int i, Vector state)"""
        return _ode.GeneralizedAlphaSolver_SetStateVector(self, i, state)
    SetStateVector = _swig_new_instance_method(_ode.GeneralizedAlphaSolver_SetStateVector)
    __swig_destroy__ = _ode.delete_GeneralizedAlphaSolver

# Register GeneralizedAlphaSolver in _ode:
_ode.GeneralizedAlphaSolver_swigregister(GeneralizedAlphaSolver)

class SIASolver(object):
    r"""Proxy of C++ mfem::SIASolver class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")

    def __init__(self, *args, **kwargs):
        raise AttributeError("No constructor defined - class is abstract")
    __repr__ = _swig_repr

    def Init(self, P, F):
        r"""Init(SIASolver self, Operator P, TimeDependentOperator F)"""
        return _ode.SIASolver_Init(self, P, F)
    Init = _swig_new_instance_method(_ode.SIASolver_Init)

    def Step(self, q, p, t, dt):
        r"""Step(SIASolver self, Vector q, Vector p, double & t, double & dt)"""
        return _ode.SIASolver_Step(self, q, p, t, dt)
    Step = _swig_new_instance_method(_ode.SIASolver_Step)

    def Run(self, q, p, t, dt, tf):
        r"""Run(SIASolver self, Vector q, Vector p, double & t, double & dt, double tf)"""
        return _ode.SIASolver_Run(self, q, p, t, dt, tf)
    Run = _swig_new_instance_method(_ode.SIASolver_Run)
    __swig_destroy__ = _ode.delete_SIASolver

# Register SIASolver in _ode:
_ode.SIASolver_swigregister(SIASolver)

class SIA1Solver(SIASolver):
    r"""Proxy of C++ mfem::SIA1Solver class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self):
        r"""__init__(SIA1Solver self) -> SIA1Solver"""
        _ode.SIA1Solver_swiginit(self, _ode.new_SIA1Solver())

    def Step(self, q, p, t, dt):
        r"""Step(SIA1Solver self, Vector q, Vector p, double & t, double & dt)"""
        return _ode.SIA1Solver_Step(self, q, p, t, dt)
    Step = _swig_new_instance_method(_ode.SIA1Solver_Step)
    __swig_destroy__ = _ode.delete_SIA1Solver

# Register SIA1Solver in _ode:
_ode.SIA1Solver_swigregister(SIA1Solver)

class SIA2Solver(SIASolver):
    r"""Proxy of C++ mfem::SIA2Solver class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self):
        r"""__init__(SIA2Solver self) -> SIA2Solver"""
        _ode.SIA2Solver_swiginit(self, _ode.new_SIA2Solver())

    def Step(self, q, p, t, dt):
        r"""Step(SIA2Solver self, Vector q, Vector p, double & t, double & dt)"""
        return _ode.SIA2Solver_Step(self, q, p, t, dt)
    Step = _swig_new_instance_method(_ode.SIA2Solver_Step)
    __swig_destroy__ = _ode.delete_SIA2Solver

# Register SIA2Solver in _ode:
_ode.SIA2Solver_swigregister(SIA2Solver)

class SIAVSolver(SIASolver):
    r"""Proxy of C++ mfem::SIAVSolver class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, order):
        r"""__init__(SIAVSolver self, int order) -> SIAVSolver"""
        _ode.SIAVSolver_swiginit(self, _ode.new_SIAVSolver(order))

    def Step(self, q, p, t, dt):
        r"""Step(SIAVSolver self, Vector q, Vector p, double & t, double & dt)"""
        return _ode.SIAVSolver_Step(self, q, p, t, dt)
    Step = _swig_new_instance_method(_ode.SIAVSolver_Step)
    __swig_destroy__ = _ode.delete_SIAVSolver

# Register SIAVSolver in _ode:
_ode.SIAVSolver_swigregister(SIAVSolver)

class SecondOrderODESolver(object):
    r"""Proxy of C++ mfem::SecondOrderODESolver class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")

    def __init__(self, *args, **kwargs):
        raise AttributeError("No constructor defined - class is abstract")
    __repr__ = _swig_repr

    def Init(self, f):
        r"""Init(SecondOrderODESolver self, SecondOrderTimeDependentOperator f)"""
        return _ode.SecondOrderODESolver_Init(self, f)
    Init = _swig_new_instance_method(_ode.SecondOrderODESolver_Init)

    def Step(self, x, dxdt, t, dt):
        r"""Step(SecondOrderODESolver self, Vector x, Vector dxdt, double & t, double & dt)"""
        return _ode.SecondOrderODESolver_Step(self, x, dxdt, t, dt)
    Step = _swig_new_instance_method(_ode.SecondOrderODESolver_Step)

    def Run(self, x, dxdt, t, dt, tf):
        r"""Run(SecondOrderODESolver self, Vector x, Vector dxdt, double & t, double & dt, double tf)"""
        return _ode.SecondOrderODESolver_Run(self, x, dxdt, t, dt, tf)
    Run = _swig_new_instance_method(_ode.SecondOrderODESolver_Run)

    def GetMaxStateSize(self):
        r"""GetMaxStateSize(SecondOrderODESolver self) -> int"""
        return _ode.SecondOrderODESolver_GetMaxStateSize(self)
    GetMaxStateSize = _swig_new_instance_method(_ode.SecondOrderODESolver_GetMaxStateSize)

    def GetStateSize(self):
        r"""GetStateSize(SecondOrderODESolver self) -> int"""
        return _ode.SecondOrderODESolver_GetStateSize(self)
    GetStateSize = _swig_new_instance_method(_ode.SecondOrderODESolver_GetStateSize)

    def GetStateVector(self, *args):
        r"""
        GetStateVector(SecondOrderODESolver self, int i) -> Vector
        GetStateVector(SecondOrderODESolver self, int i, Vector state)
        """
        return _ode.SecondOrderODESolver_GetStateVector(self, *args)
    GetStateVector = _swig_new_instance_method(_ode.SecondOrderODESolver_GetStateVector)

    def SetStateVector(self, i, state):
        r"""SetStateVector(SecondOrderODESolver self, int i, Vector state)"""
        return _ode.SecondOrderODESolver_SetStateVector(self, i, state)
    SetStateVector = _swig_new_instance_method(_ode.SecondOrderODESolver_SetStateVector)
    __swig_destroy__ = _ode.delete_SecondOrderODESolver

# Register SecondOrderODESolver in _ode:
_ode.SecondOrderODESolver_swigregister(SecondOrderODESolver)

class NewmarkSolver(SecondOrderODESolver):
    r"""Proxy of C++ mfem::NewmarkSolver class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, beta_=0.25, gamma_=0.5):
        r"""__init__(NewmarkSolver self, double beta_=0.25, double gamma_=0.5) -> NewmarkSolver"""
        _ode.NewmarkSolver_swiginit(self, _ode.new_NewmarkSolver(beta_, gamma_))

    def PrintProperties(self, *args, **kwargs):
        r"""PrintProperties(NewmarkSolver self, std::ostream & out=mfem::out)"""
        return _ode.NewmarkSolver_PrintProperties(self, *args, **kwargs)
    PrintProperties = _swig_new_instance_method(_ode.NewmarkSolver_PrintProperties)

    def Init(self, _f):
        r"""Init(NewmarkSolver self, SecondOrderTimeDependentOperator _f)"""
        return _ode.NewmarkSolver_Init(self, _f)
    Init = _swig_new_instance_method(_ode.NewmarkSolver_Init)

    def Step(self, x, dxdt, t, dt):
        r"""Step(NewmarkSolver self, Vector x, Vector dxdt, double & t, double & dt)"""
        return _ode.NewmarkSolver_Step(self, x, dxdt, t, dt)
    Step = _swig_new_instance_method(_ode.NewmarkSolver_Step)
    __swig_destroy__ = _ode.delete_NewmarkSolver

# Register NewmarkSolver in _ode:
_ode.NewmarkSolver_swigregister(NewmarkSolver)

class LinearAccelerationSolver(NewmarkSolver):
    r"""Proxy of C++ mfem::LinearAccelerationSolver class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self):
        r"""__init__(LinearAccelerationSolver self) -> LinearAccelerationSolver"""
        _ode.LinearAccelerationSolver_swiginit(self, _ode.new_LinearAccelerationSolver())
    __swig_destroy__ = _ode.delete_LinearAccelerationSolver

# Register LinearAccelerationSolver in _ode:
_ode.LinearAccelerationSolver_swigregister(LinearAccelerationSolver)

class CentralDifferenceSolver(NewmarkSolver):
    r"""Proxy of C++ mfem::CentralDifferenceSolver class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self):
        r"""__init__(CentralDifferenceSolver self) -> CentralDifferenceSolver"""
        _ode.CentralDifferenceSolver_swiginit(self, _ode.new_CentralDifferenceSolver())
    __swig_destroy__ = _ode.delete_CentralDifferenceSolver

# Register CentralDifferenceSolver in _ode:
_ode.CentralDifferenceSolver_swigregister(CentralDifferenceSolver)

class FoxGoodwinSolver(NewmarkSolver):
    r"""Proxy of C++ mfem::FoxGoodwinSolver class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self):
        r"""__init__(FoxGoodwinSolver self) -> FoxGoodwinSolver"""
        _ode.FoxGoodwinSolver_swiginit(self, _ode.new_FoxGoodwinSolver())
    __swig_destroy__ = _ode.delete_FoxGoodwinSolver

# Register FoxGoodwinSolver in _ode:
_ode.FoxGoodwinSolver_swigregister(FoxGoodwinSolver)

class GeneralizedAlpha2Solver(SecondOrderODESolver):
    r"""Proxy of C++ mfem::GeneralizedAlpha2Solver class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, rho_inf=1.0):
        r"""__init__(GeneralizedAlpha2Solver self, double rho_inf=1.0) -> GeneralizedAlpha2Solver"""
        _ode.GeneralizedAlpha2Solver_swiginit(self, _ode.new_GeneralizedAlpha2Solver(rho_inf))

    def PrintProperties(self, *args, **kwargs):
        r"""PrintProperties(GeneralizedAlpha2Solver self, std::ostream & out=mfem::out)"""
        return _ode.GeneralizedAlpha2Solver_PrintProperties(self, *args, **kwargs)
    PrintProperties = _swig_new_instance_method(_ode.GeneralizedAlpha2Solver_PrintProperties)

    def Init(self, _f):
        r"""Init(GeneralizedAlpha2Solver self, SecondOrderTimeDependentOperator _f)"""
        return _ode.GeneralizedAlpha2Solver_Init(self, _f)
    Init = _swig_new_instance_method(_ode.GeneralizedAlpha2Solver_Init)

    def Step(self, x, dxdt, t, dt):
        r"""Step(GeneralizedAlpha2Solver self, Vector x, Vector dxdt, double & t, double & dt)"""
        return _ode.GeneralizedAlpha2Solver_Step(self, x, dxdt, t, dt)
    Step = _swig_new_instance_method(_ode.GeneralizedAlpha2Solver_Step)

    def GetMaxStateSize(self):
        r"""GetMaxStateSize(GeneralizedAlpha2Solver self) -> int"""
        return _ode.GeneralizedAlpha2Solver_GetMaxStateSize(self)
    GetMaxStateSize = _swig_new_instance_method(_ode.GeneralizedAlpha2Solver_GetMaxStateSize)

    def GetStateSize(self):
        r"""GetStateSize(GeneralizedAlpha2Solver self) -> int"""
        return _ode.GeneralizedAlpha2Solver_GetStateSize(self)
    GetStateSize = _swig_new_instance_method(_ode.GeneralizedAlpha2Solver_GetStateSize)

    def GetStateVector(self, *args):
        r"""
        GetStateVector(GeneralizedAlpha2Solver self, int i) -> Vector
        GetStateVector(GeneralizedAlpha2Solver self, int i, Vector state)
        """
        return _ode.GeneralizedAlpha2Solver_GetStateVector(self, *args)
    GetStateVector = _swig_new_instance_method(_ode.GeneralizedAlpha2Solver_GetStateVector)

    def SetStateVector(self, i, state):
        r"""SetStateVector(GeneralizedAlpha2Solver self, int i, Vector state)"""
        return _ode.GeneralizedAlpha2Solver_SetStateVector(self, i, state)
    SetStateVector = _swig_new_instance_method(_ode.GeneralizedAlpha2Solver_SetStateVector)
    __swig_destroy__ = _ode.delete_GeneralizedAlpha2Solver

# Register GeneralizedAlpha2Solver in _ode:
_ode.GeneralizedAlpha2Solver_swigregister(GeneralizedAlpha2Solver)

class AverageAccelerationSolver(GeneralizedAlpha2Solver):
    r"""Proxy of C++ mfem::AverageAccelerationSolver class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self):
        r"""__init__(AverageAccelerationSolver self) -> AverageAccelerationSolver"""
        _ode.AverageAccelerationSolver_swiginit(self, _ode.new_AverageAccelerationSolver())
    __swig_destroy__ = _ode.delete_AverageAccelerationSolver

# Register AverageAccelerationSolver in _ode:
_ode.AverageAccelerationSolver_swigregister(AverageAccelerationSolver)

class HHTAlphaSolver(GeneralizedAlpha2Solver):
    r"""Proxy of C++ mfem::HHTAlphaSolver class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, alpha=1.0):
        r"""__init__(HHTAlphaSolver self, double alpha=1.0) -> HHTAlphaSolver"""
        _ode.HHTAlphaSolver_swiginit(self, _ode.new_HHTAlphaSolver(alpha))
    __swig_destroy__ = _ode.delete_HHTAlphaSolver

# Register HHTAlphaSolver in _ode:
_ode.HHTAlphaSolver_swigregister(HHTAlphaSolver)

class WBZAlphaSolver(GeneralizedAlpha2Solver):
    r"""Proxy of C++ mfem::WBZAlphaSolver class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, rho_inf=1.0):
        r"""__init__(WBZAlphaSolver self, double rho_inf=1.0) -> WBZAlphaSolver"""
        _ode.WBZAlphaSolver_swiginit(self, _ode.new_WBZAlphaSolver(rho_inf))
    __swig_destroy__ = _ode.delete_WBZAlphaSolver

# Register WBZAlphaSolver in _ode:
_ode.WBZAlphaSolver_swigregister(WBZAlphaSolver)






# This file was automatically generated by SWIG (http://www.swig.org).
# Version 4.0.2
#
# Do not make changes to this file unless you know what you are doing--modify
# the SWIG interface file instead.

from sys import version_info as _swig_python_version_info
if _swig_python_version_info < (2, 7, 0):
    raise RuntimeError("Python 2.7 or later required")

# Import the low-level C/C++ module
if __package__ or "." in __name__:
    from . import _lininteg
else:
    import _lininteg

try:
    import builtins as __builtin__
except ImportError:
    import __builtin__

_swig_new_instance_method = _lininteg.SWIG_PyInstanceMethod_New
_swig_new_static_method = _lininteg.SWIG_PyStaticMethod_New

def _swig_repr(self):
    try:
        strthis = "proxy of " + self.this.__repr__()
    except __builtin__.Exception:
        strthis = ""
    return "<%s.%s; %s >" % (self.__class__.__module__, self.__class__.__name__, strthis,)


def _swig_setattr_nondynamic_instance_variable(set):
    def set_instance_attr(self, name, value):
        if name == "thisown":
            self.this.own(value)
        elif name == "this":
            set(self, name, value)
        elif hasattr(self, name) and isinstance(getattr(type(self), name), property):
            set(self, name, value)
        else:
            raise AttributeError("You cannot add instance attributes to %s" % self)
    return set_instance_attr


def _swig_setattr_nondynamic_class_variable(set):
    def set_class_attr(cls, name, value):
        if hasattr(cls, name) and not isinstance(getattr(cls, name), property):
            set(cls, name, value)
        else:
            raise AttributeError("You cannot add class attributes to %s" % cls)
    return set_class_attr


def _swig_add_metaclass(metaclass):
    """Class decorator for adding a metaclass to a SWIG wrapped class - a slimmed down version of six.add_metaclass"""
    def wrapper(cls):
        return metaclass(cls.__name__, cls.__bases__, cls.__dict__.copy())
    return wrapper


class _SwigNonDynamicMeta(type):
    """Meta class to enforce nondynamic attributes (no new attributes) for a class"""
    __setattr__ = _swig_setattr_nondynamic_class_variable(type.__setattr__)


import weakref

import mfem._ser.fe
import mfem._ser.array
import mfem._ser.mem_manager
import mfem._ser.vector
import mfem._ser.geom
import mfem._ser.intrules
import mfem._ser.densemat
import mfem._ser.operators
import mfem._ser.matrix
import mfem._ser.sparsemat
import mfem._ser.eltrans
import mfem._ser.globals
import mfem._ser.coefficient
class LinearFormIntegrator(object):
    r"""Proxy of C++ mfem::LinearFormIntegrator class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")

    def __init__(self, *args, **kwargs):
        raise AttributeError("No constructor defined - class is abstract")
    __repr__ = _swig_repr

    def AssembleRHSElementVect(self, *args):
        r"""
        AssembleRHSElementVect(LinearFormIntegrator self, FiniteElement el, ElementTransformation Tr, Vector elvect)
        AssembleRHSElementVect(LinearFormIntegrator self, FiniteElement el, FaceElementTransformations Tr, Vector elvect)
        """
        return _lininteg.LinearFormIntegrator_AssembleRHSElementVect(self, *args)
    AssembleRHSElementVect = _swig_new_instance_method(_lininteg.LinearFormIntegrator_AssembleRHSElementVect)

    def SetIntRule(self, ir):
        r"""SetIntRule(LinearFormIntegrator self, IntegrationRule ir)"""
        return _lininteg.LinearFormIntegrator_SetIntRule(self, ir)
    SetIntRule = _swig_new_instance_method(_lininteg.LinearFormIntegrator_SetIntRule)

    def GetIntRule(self):
        r"""GetIntRule(LinearFormIntegrator self) -> IntegrationRule"""
        return _lininteg.LinearFormIntegrator_GetIntRule(self)
    GetIntRule = _swig_new_instance_method(_lininteg.LinearFormIntegrator_GetIntRule)
    __swig_destroy__ = _lininteg.delete_LinearFormIntegrator

# Register LinearFormIntegrator in _lininteg:
_lininteg.LinearFormIntegrator_swigregister(LinearFormIntegrator)

class DeltaLFIntegrator(LinearFormIntegrator):
    r"""Proxy of C++ mfem::DeltaLFIntegrator class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")

    def __init__(self, *args, **kwargs):
        raise AttributeError("No constructor defined - class is abstract")
    __repr__ = _swig_repr

    def IsDelta(self):
        r"""IsDelta(DeltaLFIntegrator self) -> bool"""
        return _lininteg.DeltaLFIntegrator_IsDelta(self)
    IsDelta = _swig_new_instance_method(_lininteg.DeltaLFIntegrator_IsDelta)

    def GetDeltaCenter(self, center):
        r"""GetDeltaCenter(DeltaLFIntegrator self, Vector center)"""
        return _lininteg.DeltaLFIntegrator_GetDeltaCenter(self, center)
    GetDeltaCenter = _swig_new_instance_method(_lininteg.DeltaLFIntegrator_GetDeltaCenter)

    def AssembleDeltaElementVect(self, fe, Trans, elvect):
        r"""AssembleDeltaElementVect(DeltaLFIntegrator self, FiniteElement fe, ElementTransformation Trans, Vector elvect)"""
        return _lininteg.DeltaLFIntegrator_AssembleDeltaElementVect(self, fe, Trans, elvect)
    AssembleDeltaElementVect = _swig_new_instance_method(_lininteg.DeltaLFIntegrator_AssembleDeltaElementVect)
    __swig_destroy__ = _lininteg.delete_DeltaLFIntegrator

# Register DeltaLFIntegrator in _lininteg:
_lininteg.DeltaLFIntegrator_swigregister(DeltaLFIntegrator)

class DomainLFIntegrator(DeltaLFIntegrator):
    r"""Proxy of C++ mfem::DomainLFIntegrator class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        r"""
        __init__(DomainLFIntegrator self, Coefficient QF, int a=2, int b=0) -> DomainLFIntegrator
        __init__(DomainLFIntegrator self, Coefficient QF, IntegrationRule ir) -> DomainLFIntegrator
        """
        _lininteg.DomainLFIntegrator_swiginit(self, _lininteg.new_DomainLFIntegrator(*args))

        self._coeff = args[0]




    def AssembleDeltaElementVect(self, fe, Trans, elvect):
        r"""AssembleDeltaElementVect(DomainLFIntegrator self, FiniteElement fe, ElementTransformation Trans, Vector elvect)"""
        return _lininteg.DomainLFIntegrator_AssembleDeltaElementVect(self, fe, Trans, elvect)
    AssembleDeltaElementVect = _swig_new_instance_method(_lininteg.DomainLFIntegrator_AssembleDeltaElementVect)

    def AssembleRHSElementVect(self, *args):
        r"""
        AssembleRHSElementVect(DomainLFIntegrator self, FiniteElement el, ElementTransformation Tr, Vector elvect)
        AssembleRHSElementVect(DomainLFIntegrator self, FiniteElement el, FaceElementTransformations Tr, Vector elvect)
        """
        return _lininteg.DomainLFIntegrator_AssembleRHSElementVect(self, *args)
    AssembleRHSElementVect = _swig_new_instance_method(_lininteg.DomainLFIntegrator_AssembleRHSElementVect)
    __swig_destroy__ = _lininteg.delete_DomainLFIntegrator

# Register DomainLFIntegrator in _lininteg:
_lininteg.DomainLFIntegrator_swigregister(DomainLFIntegrator)

class DomainLFGradIntegrator(DeltaLFIntegrator):
    r"""Proxy of C++ mfem::DomainLFGradIntegrator class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, QF):
        r"""__init__(DomainLFGradIntegrator self, VectorCoefficient QF) -> DomainLFGradIntegrator"""
        _lininteg.DomainLFGradIntegrator_swiginit(self, _lininteg.new_DomainLFGradIntegrator(QF))

    def AssembleDeltaElementVect(self, fe, Trans, elvect):
        r"""AssembleDeltaElementVect(DomainLFGradIntegrator self, FiniteElement fe, ElementTransformation Trans, Vector elvect)"""
        return _lininteg.DomainLFGradIntegrator_AssembleDeltaElementVect(self, fe, Trans, elvect)
    AssembleDeltaElementVect = _swig_new_instance_method(_lininteg.DomainLFGradIntegrator_AssembleDeltaElementVect)

    def AssembleRHSElementVect(self, *args):
        r"""
        AssembleRHSElementVect(DomainLFGradIntegrator self, FiniteElement el, ElementTransformation Tr, Vector elvect)
        AssembleRHSElementVect(DomainLFGradIntegrator self, FiniteElement el, FaceElementTransformations Tr, Vector elvect)
        """
        return _lininteg.DomainLFGradIntegrator_AssembleRHSElementVect(self, *args)
    AssembleRHSElementVect = _swig_new_instance_method(_lininteg.DomainLFGradIntegrator_AssembleRHSElementVect)
    __swig_destroy__ = _lininteg.delete_DomainLFGradIntegrator

# Register DomainLFGradIntegrator in _lininteg:
_lininteg.DomainLFGradIntegrator_swigregister(DomainLFGradIntegrator)

class BoundaryLFIntegrator(LinearFormIntegrator):
    r"""Proxy of C++ mfem::BoundaryLFIntegrator class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, QG, a=1, b=1):
        r"""__init__(BoundaryLFIntegrator self, Coefficient QG, int a=1, int b=1) -> BoundaryLFIntegrator"""
        _lininteg.BoundaryLFIntegrator_swiginit(self, _lininteg.new_BoundaryLFIntegrator(QG, a, b))

        self._coeff = QG




    def AssembleRHSElementVect(self, *args):
        r"""
        AssembleRHSElementVect(BoundaryLFIntegrator self, FiniteElement el, ElementTransformation Tr, Vector elvect)
        AssembleRHSElementVect(BoundaryLFIntegrator self, FiniteElement el, FaceElementTransformations Tr, Vector elvect)
        """
        return _lininteg.BoundaryLFIntegrator_AssembleRHSElementVect(self, *args)
    AssembleRHSElementVect = _swig_new_instance_method(_lininteg.BoundaryLFIntegrator_AssembleRHSElementVect)
    __swig_destroy__ = _lininteg.delete_BoundaryLFIntegrator

# Register BoundaryLFIntegrator in _lininteg:
_lininteg.BoundaryLFIntegrator_swigregister(BoundaryLFIntegrator)

class BoundaryNormalLFIntegrator(LinearFormIntegrator):
    r"""Proxy of C++ mfem::BoundaryNormalLFIntegrator class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, QG, a=1, b=1):
        r"""__init__(BoundaryNormalLFIntegrator self, VectorCoefficient QG, int a=1, int b=1) -> BoundaryNormalLFIntegrator"""
        _lininteg.BoundaryNormalLFIntegrator_swiginit(self, _lininteg.new_BoundaryNormalLFIntegrator(QG, a, b))

    def AssembleRHSElementVect(self, *args):
        r"""
        AssembleRHSElementVect(BoundaryNormalLFIntegrator self, FiniteElement el, ElementTransformation Tr, Vector elvect)
        AssembleRHSElementVect(BoundaryNormalLFIntegrator self, FiniteElement el, FaceElementTransformations Tr, Vector elvect)
        """
        return _lininteg.BoundaryNormalLFIntegrator_AssembleRHSElementVect(self, *args)
    AssembleRHSElementVect = _swig_new_instance_method(_lininteg.BoundaryNormalLFIntegrator_AssembleRHSElementVect)
    __swig_destroy__ = _lininteg.delete_BoundaryNormalLFIntegrator

# Register BoundaryNormalLFIntegrator in _lininteg:
_lininteg.BoundaryNormalLFIntegrator_swigregister(BoundaryNormalLFIntegrator)

class BoundaryTangentialLFIntegrator(LinearFormIntegrator):
    r"""Proxy of C++ mfem::BoundaryTangentialLFIntegrator class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, QG, a=1, b=1):
        r"""__init__(BoundaryTangentialLFIntegrator self, VectorCoefficient QG, int a=1, int b=1) -> BoundaryTangentialLFIntegrator"""
        _lininteg.BoundaryTangentialLFIntegrator_swiginit(self, _lininteg.new_BoundaryTangentialLFIntegrator(QG, a, b))

    def AssembleRHSElementVect(self, *args):
        r"""
        AssembleRHSElementVect(BoundaryTangentialLFIntegrator self, FiniteElement el, ElementTransformation Tr, Vector elvect)
        AssembleRHSElementVect(BoundaryTangentialLFIntegrator self, FiniteElement el, FaceElementTransformations Tr, Vector elvect)
        """
        return _lininteg.BoundaryTangentialLFIntegrator_AssembleRHSElementVect(self, *args)
    AssembleRHSElementVect = _swig_new_instance_method(_lininteg.BoundaryTangentialLFIntegrator_AssembleRHSElementVect)
    __swig_destroy__ = _lininteg.delete_BoundaryTangentialLFIntegrator

# Register BoundaryTangentialLFIntegrator in _lininteg:
_lininteg.BoundaryTangentialLFIntegrator_swigregister(BoundaryTangentialLFIntegrator)

class VectorDomainLFIntegrator(DeltaLFIntegrator):
    r"""Proxy of C++ mfem::VectorDomainLFIntegrator class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, QF):
        r"""__init__(VectorDomainLFIntegrator self, VectorCoefficient QF) -> VectorDomainLFIntegrator"""
        _lininteg.VectorDomainLFIntegrator_swiginit(self, _lininteg.new_VectorDomainLFIntegrator(QF))

    def AssembleDeltaElementVect(self, fe, Trans, elvect):
        r"""AssembleDeltaElementVect(VectorDomainLFIntegrator self, FiniteElement fe, ElementTransformation Trans, Vector elvect)"""
        return _lininteg.VectorDomainLFIntegrator_AssembleDeltaElementVect(self, fe, Trans, elvect)
    AssembleDeltaElementVect = _swig_new_instance_method(_lininteg.VectorDomainLFIntegrator_AssembleDeltaElementVect)

    def AssembleRHSElementVect(self, *args):
        r"""
        AssembleRHSElementVect(VectorDomainLFIntegrator self, FiniteElement el, ElementTransformation Tr, Vector elvect)
        AssembleRHSElementVect(VectorDomainLFIntegrator self, FiniteElement el, FaceElementTransformations Tr, Vector elvect)
        """
        return _lininteg.VectorDomainLFIntegrator_AssembleRHSElementVect(self, *args)
    AssembleRHSElementVect = _swig_new_instance_method(_lininteg.VectorDomainLFIntegrator_AssembleRHSElementVect)
    __swig_destroy__ = _lininteg.delete_VectorDomainLFIntegrator

# Register VectorDomainLFIntegrator in _lininteg:
_lininteg.VectorDomainLFIntegrator_swigregister(VectorDomainLFIntegrator)

class VectorBoundaryLFIntegrator(LinearFormIntegrator):
    r"""Proxy of C++ mfem::VectorBoundaryLFIntegrator class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, QG):
        r"""__init__(VectorBoundaryLFIntegrator self, VectorCoefficient QG) -> VectorBoundaryLFIntegrator"""
        _lininteg.VectorBoundaryLFIntegrator_swiginit(self, _lininteg.new_VectorBoundaryLFIntegrator(QG))

    def AssembleRHSElementVect(self, *args):
        r"""
        AssembleRHSElementVect(VectorBoundaryLFIntegrator self, FiniteElement el, ElementTransformation Tr, Vector elvect)
        AssembleRHSElementVect(VectorBoundaryLFIntegrator self, FiniteElement el, FaceElementTransformations Tr, Vector elvect)
        """
        return _lininteg.VectorBoundaryLFIntegrator_AssembleRHSElementVect(self, *args)
    AssembleRHSElementVect = _swig_new_instance_method(_lininteg.VectorBoundaryLFIntegrator_AssembleRHSElementVect)
    __swig_destroy__ = _lininteg.delete_VectorBoundaryLFIntegrator

# Register VectorBoundaryLFIntegrator in _lininteg:
_lininteg.VectorBoundaryLFIntegrator_swigregister(VectorBoundaryLFIntegrator)

class VectorFEDomainLFIntegrator(DeltaLFIntegrator):
    r"""Proxy of C++ mfem::VectorFEDomainLFIntegrator class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, F):
        r"""__init__(VectorFEDomainLFIntegrator self, VectorCoefficient F) -> VectorFEDomainLFIntegrator"""
        _lininteg.VectorFEDomainLFIntegrator_swiginit(self, _lininteg.new_VectorFEDomainLFIntegrator(F))

        self._coeff = F




    def AssembleDeltaElementVect(self, fe, Trans, elvect):
        r"""AssembleDeltaElementVect(VectorFEDomainLFIntegrator self, FiniteElement fe, ElementTransformation Trans, Vector elvect)"""
        return _lininteg.VectorFEDomainLFIntegrator_AssembleDeltaElementVect(self, fe, Trans, elvect)
    AssembleDeltaElementVect = _swig_new_instance_method(_lininteg.VectorFEDomainLFIntegrator_AssembleDeltaElementVect)

    def AssembleRHSElementVect(self, *args):
        r"""
        AssembleRHSElementVect(VectorFEDomainLFIntegrator self, FiniteElement el, ElementTransformation Tr, Vector elvect)
        AssembleRHSElementVect(VectorFEDomainLFIntegrator self, FiniteElement el, FaceElementTransformations Tr, Vector elvect)
        """
        return _lininteg.VectorFEDomainLFIntegrator_AssembleRHSElementVect(self, *args)
    AssembleRHSElementVect = _swig_new_instance_method(_lininteg.VectorFEDomainLFIntegrator_AssembleRHSElementVect)
    __swig_destroy__ = _lininteg.delete_VectorFEDomainLFIntegrator

# Register VectorFEDomainLFIntegrator in _lininteg:
_lininteg.VectorFEDomainLFIntegrator_swigregister(VectorFEDomainLFIntegrator)

class VectorFEDomainLFCurlIntegrator(DeltaLFIntegrator):
    r"""Proxy of C++ mfem::VectorFEDomainLFCurlIntegrator class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, F):
        r"""__init__(VectorFEDomainLFCurlIntegrator self, VectorCoefficient F) -> VectorFEDomainLFCurlIntegrator"""
        _lininteg.VectorFEDomainLFCurlIntegrator_swiginit(self, _lininteg.new_VectorFEDomainLFCurlIntegrator(F))

    def AssembleDeltaElementVect(self, fe, Trans, elvect):
        r"""AssembleDeltaElementVect(VectorFEDomainLFCurlIntegrator self, FiniteElement fe, ElementTransformation Trans, Vector elvect)"""
        return _lininteg.VectorFEDomainLFCurlIntegrator_AssembleDeltaElementVect(self, fe, Trans, elvect)
    AssembleDeltaElementVect = _swig_new_instance_method(_lininteg.VectorFEDomainLFCurlIntegrator_AssembleDeltaElementVect)

    def AssembleRHSElementVect(self, *args):
        r"""
        AssembleRHSElementVect(VectorFEDomainLFCurlIntegrator self, FiniteElement el, ElementTransformation Tr, Vector elvect)
        AssembleRHSElementVect(VectorFEDomainLFCurlIntegrator self, FiniteElement el, FaceElementTransformations Tr, Vector elvect)
        """
        return _lininteg.VectorFEDomainLFCurlIntegrator_AssembleRHSElementVect(self, *args)
    AssembleRHSElementVect = _swig_new_instance_method(_lininteg.VectorFEDomainLFCurlIntegrator_AssembleRHSElementVect)
    __swig_destroy__ = _lininteg.delete_VectorFEDomainLFCurlIntegrator

# Register VectorFEDomainLFCurlIntegrator in _lininteg:
_lininteg.VectorFEDomainLFCurlIntegrator_swigregister(VectorFEDomainLFCurlIntegrator)

class VectorFEDomainLFDivIntegrator(DeltaLFIntegrator):
    r"""Proxy of C++ mfem::VectorFEDomainLFDivIntegrator class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, QF):
        r"""__init__(VectorFEDomainLFDivIntegrator self, Coefficient QF) -> VectorFEDomainLFDivIntegrator"""
        _lininteg.VectorFEDomainLFDivIntegrator_swiginit(self, _lininteg.new_VectorFEDomainLFDivIntegrator(QF))

    def AssembleDeltaElementVect(self, fe, Trans, elvect):
        r"""AssembleDeltaElementVect(VectorFEDomainLFDivIntegrator self, FiniteElement fe, ElementTransformation Trans, Vector elvect)"""
        return _lininteg.VectorFEDomainLFDivIntegrator_AssembleDeltaElementVect(self, fe, Trans, elvect)
    AssembleDeltaElementVect = _swig_new_instance_method(_lininteg.VectorFEDomainLFDivIntegrator_AssembleDeltaElementVect)

    def AssembleRHSElementVect(self, *args):
        r"""
        AssembleRHSElementVect(VectorFEDomainLFDivIntegrator self, FiniteElement el, ElementTransformation Tr, Vector elvect)
        AssembleRHSElementVect(VectorFEDomainLFDivIntegrator self, FiniteElement el, FaceElementTransformations Tr, Vector elvect)
        """
        return _lininteg.VectorFEDomainLFDivIntegrator_AssembleRHSElementVect(self, *args)
    AssembleRHSElementVect = _swig_new_instance_method(_lininteg.VectorFEDomainLFDivIntegrator_AssembleRHSElementVect)
    __swig_destroy__ = _lininteg.delete_VectorFEDomainLFDivIntegrator

# Register VectorFEDomainLFDivIntegrator in _lininteg:
_lininteg.VectorFEDomainLFDivIntegrator_swigregister(VectorFEDomainLFDivIntegrator)

class VectorBoundaryFluxLFIntegrator(LinearFormIntegrator):
    r"""Proxy of C++ mfem::VectorBoundaryFluxLFIntegrator class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, f, s=1.0, ir=None):
        r"""__init__(VectorBoundaryFluxLFIntegrator self, Coefficient f, double s=1.0, IntegrationRule ir=None) -> VectorBoundaryFluxLFIntegrator"""
        _lininteg.VectorBoundaryFluxLFIntegrator_swiginit(self, _lininteg.new_VectorBoundaryFluxLFIntegrator(f, s, ir))

    def AssembleRHSElementVect(self, *args):
        r"""
        AssembleRHSElementVect(VectorBoundaryFluxLFIntegrator self, FiniteElement el, ElementTransformation Tr, Vector elvect)
        AssembleRHSElementVect(VectorBoundaryFluxLFIntegrator self, FiniteElement el, FaceElementTransformations Tr, Vector elvect)
        """
        return _lininteg.VectorBoundaryFluxLFIntegrator_AssembleRHSElementVect(self, *args)
    AssembleRHSElementVect = _swig_new_instance_method(_lininteg.VectorBoundaryFluxLFIntegrator_AssembleRHSElementVect)
    __swig_destroy__ = _lininteg.delete_VectorBoundaryFluxLFIntegrator

# Register VectorBoundaryFluxLFIntegrator in _lininteg:
_lininteg.VectorBoundaryFluxLFIntegrator_swigregister(VectorBoundaryFluxLFIntegrator)

class VectorFEBoundaryFluxLFIntegrator(LinearFormIntegrator):
    r"""Proxy of C++ mfem::VectorFEBoundaryFluxLFIntegrator class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        r"""
        __init__(VectorFEBoundaryFluxLFIntegrator self, int a=1, int b=-1) -> VectorFEBoundaryFluxLFIntegrator
        __init__(VectorFEBoundaryFluxLFIntegrator self, Coefficient f, int a=2, int b=0) -> VectorFEBoundaryFluxLFIntegrator
        """
        _lininteg.VectorFEBoundaryFluxLFIntegrator_swiginit(self, _lininteg.new_VectorFEBoundaryFluxLFIntegrator(*args))

    def AssembleRHSElementVect(self, *args):
        r"""
        AssembleRHSElementVect(VectorFEBoundaryFluxLFIntegrator self, FiniteElement el, ElementTransformation Tr, Vector elvect)
        AssembleRHSElementVect(VectorFEBoundaryFluxLFIntegrator self, FiniteElement el, FaceElementTransformations Tr, Vector elvect)
        """
        return _lininteg.VectorFEBoundaryFluxLFIntegrator_AssembleRHSElementVect(self, *args)
    AssembleRHSElementVect = _swig_new_instance_method(_lininteg.VectorFEBoundaryFluxLFIntegrator_AssembleRHSElementVect)
    __swig_destroy__ = _lininteg.delete_VectorFEBoundaryFluxLFIntegrator

# Register VectorFEBoundaryFluxLFIntegrator in _lininteg:
_lininteg.VectorFEBoundaryFluxLFIntegrator_swigregister(VectorFEBoundaryFluxLFIntegrator)

class VectorFEBoundaryTangentLFIntegrator(LinearFormIntegrator):
    r"""Proxy of C++ mfem::VectorFEBoundaryTangentLFIntegrator class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, QG, a=2, b=0):
        r"""__init__(VectorFEBoundaryTangentLFIntegrator self, VectorCoefficient QG, int a=2, int b=0) -> VectorFEBoundaryTangentLFIntegrator"""
        _lininteg.VectorFEBoundaryTangentLFIntegrator_swiginit(self, _lininteg.new_VectorFEBoundaryTangentLFIntegrator(QG, a, b))

        self._coeff = QG




    def AssembleRHSElementVect(self, *args):
        r"""
        AssembleRHSElementVect(VectorFEBoundaryTangentLFIntegrator self, FiniteElement el, ElementTransformation Tr, Vector elvect)
        AssembleRHSElementVect(VectorFEBoundaryTangentLFIntegrator self, FiniteElement el, FaceElementTransformations Tr, Vector elvect)
        """
        return _lininteg.VectorFEBoundaryTangentLFIntegrator_AssembleRHSElementVect(self, *args)
    AssembleRHSElementVect = _swig_new_instance_method(_lininteg.VectorFEBoundaryTangentLFIntegrator_AssembleRHSElementVect)
    __swig_destroy__ = _lininteg.delete_VectorFEBoundaryTangentLFIntegrator

# Register VectorFEBoundaryTangentLFIntegrator in _lininteg:
_lininteg.VectorFEBoundaryTangentLFIntegrator_swigregister(VectorFEBoundaryTangentLFIntegrator)

class BoundaryFlowIntegrator(LinearFormIntegrator):
    r"""Proxy of C++ mfem::BoundaryFlowIntegrator class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, _f, _u, a, b):
        r"""__init__(BoundaryFlowIntegrator self, Coefficient _f, VectorCoefficient _u, double a, double b) -> BoundaryFlowIntegrator"""
        _lininteg.BoundaryFlowIntegrator_swiginit(self, _lininteg.new_BoundaryFlowIntegrator(_f, _u, a, b))

    def AssembleRHSElementVect(self, *args):
        r"""
        AssembleRHSElementVect(BoundaryFlowIntegrator self, FiniteElement el, ElementTransformation Tr, Vector elvect)
        AssembleRHSElementVect(BoundaryFlowIntegrator self, FiniteElement el, FaceElementTransformations Tr, Vector elvect)
        """
        return _lininteg.BoundaryFlowIntegrator_AssembleRHSElementVect(self, *args)
    AssembleRHSElementVect = _swig_new_instance_method(_lininteg.BoundaryFlowIntegrator_AssembleRHSElementVect)
    __swig_destroy__ = _lininteg.delete_BoundaryFlowIntegrator

# Register BoundaryFlowIntegrator in _lininteg:
_lininteg.BoundaryFlowIntegrator_swigregister(BoundaryFlowIntegrator)

class DGDirichletLFIntegrator(LinearFormIntegrator):
    r"""Proxy of C++ mfem::DGDirichletLFIntegrator class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        r"""
        __init__(DGDirichletLFIntegrator self, Coefficient u, double const s, double const k) -> DGDirichletLFIntegrator
        __init__(DGDirichletLFIntegrator self, Coefficient u, Coefficient q, double const s, double const k) -> DGDirichletLFIntegrator
        __init__(DGDirichletLFIntegrator self, Coefficient u, MatrixCoefficient q, double const s, double const k) -> DGDirichletLFIntegrator
        """
        _lininteg.DGDirichletLFIntegrator_swiginit(self, _lininteg.new_DGDirichletLFIntegrator(*args))

    def AssembleRHSElementVect(self, *args):
        r"""
        AssembleRHSElementVect(DGDirichletLFIntegrator self, FiniteElement el, ElementTransformation Tr, Vector elvect)
        AssembleRHSElementVect(DGDirichletLFIntegrator self, FiniteElement el, FaceElementTransformations Tr, Vector elvect)
        """
        return _lininteg.DGDirichletLFIntegrator_AssembleRHSElementVect(self, *args)
    AssembleRHSElementVect = _swig_new_instance_method(_lininteg.DGDirichletLFIntegrator_AssembleRHSElementVect)
    __swig_destroy__ = _lininteg.delete_DGDirichletLFIntegrator

# Register DGDirichletLFIntegrator in _lininteg:
_lininteg.DGDirichletLFIntegrator_swigregister(DGDirichletLFIntegrator)

class DGElasticityDirichletLFIntegrator(LinearFormIntegrator):
    r"""Proxy of C++ mfem::DGElasticityDirichletLFIntegrator class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, uD_, lambda_, mu_, alpha_, kappa_):
        r"""__init__(DGElasticityDirichletLFIntegrator self, VectorCoefficient uD_, Coefficient lambda_, Coefficient mu_, double alpha_, double kappa_) -> DGElasticityDirichletLFIntegrator"""
        _lininteg.DGElasticityDirichletLFIntegrator_swiginit(self, _lininteg.new_DGElasticityDirichletLFIntegrator(uD_, lambda_, mu_, alpha_, kappa_))

    def AssembleRHSElementVect(self, *args):
        r"""
        AssembleRHSElementVect(DGElasticityDirichletLFIntegrator self, FiniteElement el, ElementTransformation Tr, Vector elvect)
        AssembleRHSElementVect(DGElasticityDirichletLFIntegrator self, FiniteElement el, FaceElementTransformations Tr, Vector elvect)
        """
        return _lininteg.DGElasticityDirichletLFIntegrator_AssembleRHSElementVect(self, *args)
    AssembleRHSElementVect = _swig_new_instance_method(_lininteg.DGElasticityDirichletLFIntegrator_AssembleRHSElementVect)
    __swig_destroy__ = _lininteg.delete_DGElasticityDirichletLFIntegrator

# Register DGElasticityDirichletLFIntegrator in _lininteg:
_lininteg.DGElasticityDirichletLFIntegrator_swigregister(DGElasticityDirichletLFIntegrator)

class VectorQuadratureLFIntegrator(LinearFormIntegrator):
    r"""Proxy of C++ mfem::VectorQuadratureLFIntegrator class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")

    def __init__(self, *args, **kwargs):
        raise AttributeError("No constructor defined - class is abstract")
    __repr__ = _swig_repr

    def AssembleRHSElementVect(self, *args):
        r"""
        AssembleRHSElementVect(VectorQuadratureLFIntegrator self, FiniteElement el, ElementTransformation Tr, Vector elvect)
        AssembleRHSElementVect(VectorQuadratureLFIntegrator self, FiniteElement el, FaceElementTransformations Tr, Vector elvect)
        AssembleRHSElementVect(VectorQuadratureLFIntegrator self, FiniteElement fe, ElementTransformation Tr, Vector elvect)
        """
        return _lininteg.VectorQuadratureLFIntegrator_AssembleRHSElementVect(self, *args)
    AssembleRHSElementVect = _swig_new_instance_method(_lininteg.VectorQuadratureLFIntegrator_AssembleRHSElementVect)

    def SetIntRule(self, ir):
        r"""SetIntRule(VectorQuadratureLFIntegrator self, IntegrationRule ir)"""
        return _lininteg.VectorQuadratureLFIntegrator_SetIntRule(self, ir)
    SetIntRule = _swig_new_instance_method(_lininteg.VectorQuadratureLFIntegrator_SetIntRule)
    __swig_destroy__ = _lininteg.delete_VectorQuadratureLFIntegrator

# Register VectorQuadratureLFIntegrator in _lininteg:
_lininteg.VectorQuadratureLFIntegrator_swigregister(VectorQuadratureLFIntegrator)

class QuadratureLFIntegrator(LinearFormIntegrator):
    r"""Proxy of C++ mfem::QuadratureLFIntegrator class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")

    def __init__(self, *args, **kwargs):
        raise AttributeError("No constructor defined - class is abstract")
    __repr__ = _swig_repr

    def AssembleRHSElementVect(self, *args):
        r"""
        AssembleRHSElementVect(QuadratureLFIntegrator self, FiniteElement el, ElementTransformation Tr, Vector elvect)
        AssembleRHSElementVect(QuadratureLFIntegrator self, FiniteElement el, FaceElementTransformations Tr, Vector elvect)
        AssembleRHSElementVect(QuadratureLFIntegrator self, FiniteElement fe, ElementTransformation Tr, Vector elvect)
        """
        return _lininteg.QuadratureLFIntegrator_AssembleRHSElementVect(self, *args)
    AssembleRHSElementVect = _swig_new_instance_method(_lininteg.QuadratureLFIntegrator_AssembleRHSElementVect)

    def SetIntRule(self, ir):
        r"""SetIntRule(QuadratureLFIntegrator self, IntegrationRule ir)"""
        return _lininteg.QuadratureLFIntegrator_SetIntRule(self, ir)
    SetIntRule = _swig_new_instance_method(_lininteg.QuadratureLFIntegrator_SetIntRule)
    __swig_destroy__ = _lininteg.delete_QuadratureLFIntegrator

# Register QuadratureLFIntegrator in _lininteg:
_lininteg.QuadratureLFIntegrator_swigregister(QuadratureLFIntegrator)






from  mfem.ser.array import *
from  mfem.ser.mesh import *
from  mfem.ser.point import *
from  mfem.ser.segment import *
from  mfem.ser.common_functions import *
from  mfem.ser.operators import *
from  mfem.ser.blockoperator import *
from  mfem.ser.blockvector import *
from  mfem.ser.blockmatrix import *
from  mfem.ser.coefficient import *
from  mfem.ser.lininteg import *
from  mfem.ser.socketstream import *

from  mfem.ser.fe_coll import *
from  mfem.ser.vector import *
from  mfem.ser.complex_operator import *
from  mfem.ser.fespace import *
from  mfem.ser.linearform import *
from  mfem.ser.bilininteg import *
from  mfem.ser.gridfunc import *
from  mfem.ser.intrules import *
from  mfem.ser.fe import *
from  mfem.ser.ode import *
from  mfem.ser.bilinearform import *
from  mfem.ser.estimators import *
from  mfem.ser.mesh_operators import *
from  mfem.ser.sparsemat import *
from  mfem.ser.densemat import *
from  mfem.ser.solvers import *
from  mfem.ser.sparsesmoothers import *
from  mfem.ser.eltrans import *
from  mfem.ser.geom import *
from  mfem.ser.vertex import *
from  mfem.ser.table import *
from  mfem.ser.element import *
from  mfem.ser.nonlininteg import *
from  mfem.ser.nonlinearform import *




'''
   MFEM example 1 (converted from ex1.cpp)

   See c++ version in the MFEM library for more detail 

   How to run:
      python <arguments>

   Example of arguments:
      ex1.py -m star.mesh
      ex1.py -m square-disc.mesh
      ex1.py -m escher.mesh
      ex1.py -m fichera.mesh
      ex1.py -m square-disc-p3.mesh -o 3
      ex1.py -m square-disc-nurbs.mesh -o -1
      ex1.py -m disc-nurbs.mesh -o -1
      ex1.py -m pipe-nurbs.mesh -o -1
      ex1.py -m star-surf.mesh
      ex1.py -m square-disc-surf.mesh
      ex1.py -m inline-segment.mesh
      ex1.py -m amr-quad.mesh
      ex1.py -m amr-hex.mesh
      ex1.py -m fichera-amr.mesh
      ex1.py -m mobius-strip.mesh
      ex1.py -m mobius-strip.mesh -o -1 -sc

   Description:  This example code demonstrates the use of MFEM to define a
                 simple finite element discretization of the Laplace problem
                 -Delta u = 1 with homogeneous Dirichlet boundary conditions.

'''

import sys
from os.path import expanduser, join
import numpy as np

from mfem.common.arg_parser import ArgParser
from mfem import path
import mfem.ser as mfem

import os
def_meshfile =expanduser(join(os.path.dirname(__file__), '..', 'data', 'star.mesh'))

def run(order = 1, static_cond = False,
        meshfile = def_meshfile, visualization = False,
        device='cpu', pa=False):

   device = mfem.Device(device)
   device.Print()
   
   mesh = mfem.Mesh(meshfile, 1,1)
   dim = mesh.Dimension()

   #   3. Refine the mesh to increase the resolution. In this example we do
   #      'ref_levels' of uniform refinement. We choose 'ref_levels' to be the
   #      largest number that gives a final mesh with no more than 50,000
   #      elements.
   ref_levels = int(np.floor(np.log(50000./mesh.GetNE())/np.log(2.)/dim))
   for x in range(ref_levels):
      mesh.UniformRefinement();

   #5. Define a finite element space on the mesh. Here we use vector finite
   #   elements, i.e. dim copies of a scalar finite element space. The vector
   #   dimension is specified by the last argument of the FiniteElementSpace
   #   constructor. For NURBS meshes, we use the (degree elevated) NURBS space
   #   associated with the mesh nodes.
   if order > 0:
       fec = mfem.H1_FECollection(order, dim)
   elif mesh.GetNodes():
       fec = mesh.GetNodes().OwnFEC()
       print( "Using isoparametric FEs: " + str(fec.Name()));
   else:
       order = 1
       fec = mfem.H1_FECollection(order, dim)
   fespace = mfem.FiniteElementSpace(mesh, fec)
   print('Number of finite element unknowns: '+
          str(fespace.GetTrueVSize()))
   # 5. Determine the list of true (i.e. conforming) essential boundary dofs.
   #    In this example, the boundary conditions are defined by marking all
   #    the boundary attributes from the mesh as essential (Dirichlet) and
   #    converting them to a list of true dofs.
   ess_tdof_list = mfem.intArray()
   if mesh.bdr_attributes.Size()>0:
       ess_bdr = mfem.intArray([1]*mesh.bdr_attributes.Max())
       ess_bdr = mfem.intArray(mesh.bdr_attributes.Max())
       ess_bdr.Assign(1)
       fespace.GetEssentialTrueDofs(ess_bdr, ess_tdof_list)
   #6. Set up the linear form b(.) which corresponds to the right-hand side of
   #   the FEM linear system, which in this case is (1,phi_i) where phi_i are
   #   the basis functions in the finite element fespace.
   b = mfem.LinearForm(fespace)
   one = mfem.ConstantCoefficient(1.0)
   b.AddDomainIntegrator(mfem.DomainLFIntegrator(one))
   b.Assemble();
   #7. Define the solution vector x as a finite element grid function
   #   corresponding to fespace. Initialize x with initial guess of zero,
   #   which satisfies the boundary conditions.
   x = mfem.GridFunction(fespace);
   x.Assign(0.0)
   #8. Set up the bilinear form a(.,.) on the finite element space
   #   corresponding to the Laplacian operator -Delta, by adding the Diffusion
   #   domain integrator.
   a = mfem.BilinearForm(fespace);
   if pa:
       a.SetAssemblyLevel(mfem.AssemblyLevel_PARTIAL)
   a.AddDomainIntegrator(mfem.DiffusionIntegrator(one))
   #9. Assemble the bilinear form and the corresponding linear system,
   #   applying any necessary transformations such as: eliminating boundary
   #   conditions, applying conforming constraints for non-conforming AMR,
   #   static condensation, etc.
   if static_cond: a.EnableStaticCondensation()
   a.Assemble();

   A = mfem.OperatorPtr()
   B = mfem.Vector()
   X = mfem.Vector()
   
   a.FormLinearSystem(ess_tdof_list, x, b, A, X, B);
   print("Size of linear system: " + str(A.Height()))
   
   # 10. Solve

   if pa:
       if mfem.UsesTensorBasis(fespace):
           M = mfem.OperatorJacobiSmoother(a, ess_tdof_list)
           mfem.PCG(A, M, B, X, 1, 4000, 1e-12, 0.0)
       else:
           mfem.CG(A, B, X, 1, 400, 1e-12, 0.0);
   else:
       AA = mfem.OperatorHandle2SparseMatrix(A)      
       M = mfem.GSSmoother(AA)
       mfem.PCG(A, M, B, X, 1, 200, 1e-12, 0.0);

   # 11. Recover the solution as a finite element grid function.
   a.RecoverFEMSolution(X, b, x)
   # 12. Save the refined mesh and the solution. This output can be viewed later
   #     using GLVis: "glvis -m refined.mesh -g sol.gf".
   mesh.Print('refined.mesh', 8)
   x.Save('sol.gf', 8)

   #13. Send the solution by socket to a GLVis server.
   if (visualization):
       sol_sock = mfem.socketstream("localhost", 19916)
       sol_sock.precision(8)
       sol_sock.send_solution(mesh,  x)


if __name__ == "__main__":
   parser = ArgParser(description='Ex1 (Laplace Problem)')
   parser.add_argument('-m', '--mesh',
                       default = 'star.mesh',
                       action = 'store', type = str,
                       help='Mesh file to use.')
   parser.add_argument('-vis', '--visualization',
                       action = 'store_true',
                       help='Enable GLVis visualization')
   parser.add_argument('-o', '--order',
                       action = 'store', default = 1, type=int,
                       help = "Finite element order (polynomial degree) or -1 for isoparametric space.");
   parser.add_argument('-sc', '--static-condensation',
                       action = 'store_true', 
                       help = "Enable static condensation.")
   parser.add_argument("-pa", "--partial-assembly",
                       action = 'store_true',
                       help = "Enable Partial Assembly.");
   parser.add_argument("-d", "--device",
                       default = "cpu", type=str, 
                       help = "Device configuration string, see Device::Configure().")
   
   args = parser.parse_args()
   parser.print_options(args)


   order = args.order
   static_cond = args.static_condensation

   meshfile = expanduser(join(os.path.dirname(__file__), '..', 'data', args.mesh))
   visualization = args.visualization
   device = args.device
   pa = args.partial_assembly

   run(order=order,
       static_cond=static_cond,
       meshfile=meshfile,
       visualization=visualization,
       device=device,
       pa=pa)




# This file was automatically generated by SWIG (http://www.swig.org).
# Version 4.0.2
#
# Do not make changes to this file unless you know what you are doing--modify
# the SWIG interface file instead.

from sys import version_info as _swig_python_version_info
if _swig_python_version_info < (2, 7, 0):
    raise RuntimeError("Python 2.7 or later required")

# Import the low-level C/C++ module
if __package__ or "." in __name__:
    from . import _device
else:
    import _device

try:
    import builtins as __builtin__
except ImportError:
    import __builtin__

_swig_new_instance_method = _device.SWIG_PyInstanceMethod_New
_swig_new_static_method = _device.SWIG_PyStaticMethod_New

def _swig_repr(self):
    try:
        strthis = "proxy of " + self.this.__repr__()
    except __builtin__.Exception:
        strthis = ""
    return "<%s.%s; %s >" % (self.__class__.__module__, self.__class__.__name__, strthis,)


def _swig_setattr_nondynamic_instance_variable(set):
    def set_instance_attr(self, name, value):
        if name == "thisown":
            self.this.own(value)
        elif name == "this":
            set(self, name, value)
        elif hasattr(self, name) and isinstance(getattr(type(self), name), property):
            set(self, name, value)
        else:
            raise AttributeError("You cannot add instance attributes to %s" % self)
    return set_instance_attr


def _swig_setattr_nondynamic_class_variable(set):
    def set_class_attr(cls, name, value):
        if hasattr(cls, name) and not isinstance(getattr(cls, name), property):
            set(cls, name, value)
        else:
            raise AttributeError("You cannot add class attributes to %s" % cls)
    return set_class_attr


def _swig_add_metaclass(metaclass):
    """Class decorator for adding a metaclass to a SWIG wrapped class - a slimmed down version of six.add_metaclass"""
    def wrapper(cls):
        return metaclass(cls.__name__, cls.__bases__, cls.__dict__.copy())
    return wrapper


class _SwigNonDynamicMeta(type):
    """Meta class to enforce nondynamic attributes (no new attributes) for a class"""
    __setattr__ = _swig_setattr_nondynamic_class_variable(type.__setattr__)


import mfem._par.mem_manager
class Backend(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    CPU = _device.Backend_CPU
    OMP = _device.Backend_OMP
    CUDA = _device.Backend_CUDA
    HIP = _device.Backend_HIP
    RAJA_CPU = _device.Backend_RAJA_CPU
    RAJA_OMP = _device.Backend_RAJA_OMP
    RAJA_CUDA = _device.Backend_RAJA_CUDA
    OCCA_CPU = _device.Backend_OCCA_CPU
    OCCA_OMP = _device.Backend_OCCA_OMP
    OCCA_CUDA = _device.Backend_OCCA_CUDA
    CEED_CPU = _device.Backend_CEED_CPU
    CEED_CUDA = _device.Backend_CEED_CUDA
    CEED_HIP = _device.Backend_CEED_HIP
    DEBUG_DEVICE = _device.Backend_DEBUG_DEVICE
    NUM_BACKENDS = _device.Backend_NUM_BACKENDS
    CPU_MASK = _device.Backend_CPU_MASK
    CUDA_MASK = _device.Backend_CUDA_MASK
    HIP_MASK = _device.Backend_HIP_MASK
    OMP_MASK = _device.Backend_OMP_MASK
    CEED_MASK = _device.Backend_CEED_MASK
    DEVICE_MASK = _device.Backend_DEVICE_MASK
    RAJA_MASK = _device.Backend_RAJA_MASK
    OCCA_MASK = _device.Backend_OCCA_MASK

    def __init__(self):
        _device.Backend_swiginit(self, _device.new_Backend())
    __swig_destroy__ = _device.delete_Backend

# Register Backend in _device:
_device.Backend_swigregister(Backend)

class Device(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        _device.Device_swiginit(self, _device.new_Device(*args))
    __swig_destroy__ = _device.delete_Device

    def Configure(self, device, dev=0):
        return _device.Device_Configure(self, device, dev)
    Configure = _swig_new_instance_method(_device.Device_Configure)

    def Print(self, *args, **kwargs):
        return _device.Device_Print(self, *args, **kwargs)
    Print = _swig_new_instance_method(_device.Device_Print)

    @staticmethod
    def IsConfigured():
        return _device.Device_IsConfigured()
    IsConfigured = _swig_new_static_method(_device.Device_IsConfigured)

    @staticmethod
    def IsAvailable():
        return _device.Device_IsAvailable()
    IsAvailable = _swig_new_static_method(_device.Device_IsAvailable)

    @staticmethod
    def IsEnabled():
        return _device.Device_IsEnabled()
    IsEnabled = _swig_new_static_method(_device.Device_IsEnabled)

    @staticmethod
    def IsDisabled():
        return _device.Device_IsDisabled()
    IsDisabled = _swig_new_static_method(_device.Device_IsDisabled)

    @staticmethod
    def Allows(b_mask):
        return _device.Device_Allows(b_mask)
    Allows = _swig_new_static_method(_device.Device_Allows)

    @staticmethod
    def GetHostMemoryType():
        return _device.Device_GetHostMemoryType()
    GetHostMemoryType = _swig_new_static_method(_device.Device_GetHostMemoryType)

    @staticmethod
    def GetHostMemoryClass():
        return _device.Device_GetHostMemoryClass()
    GetHostMemoryClass = _swig_new_static_method(_device.Device_GetHostMemoryClass)

    @staticmethod
    def GetDeviceMemoryType():
        return _device.Device_GetDeviceMemoryType()
    GetDeviceMemoryType = _swig_new_static_method(_device.Device_GetDeviceMemoryType)

    @staticmethod
    def GetMemoryType():
        return _device.Device_GetMemoryType()
    GetMemoryType = _swig_new_static_method(_device.Device_GetMemoryType)

    @staticmethod
    def GetDeviceMemoryClass():
        return _device.Device_GetDeviceMemoryClass()
    GetDeviceMemoryClass = _swig_new_static_method(_device.Device_GetDeviceMemoryClass)

    @staticmethod
    def GetMemoryClass():
        return _device.Device_GetMemoryClass()
    GetMemoryClass = _swig_new_static_method(_device.Device_GetMemoryClass)

    @staticmethod
    def SetGPUAwareMPI(force=True):
        return _device.Device_SetGPUAwareMPI(force)
    SetGPUAwareMPI = _swig_new_static_method(_device.Device_SetGPUAwareMPI)

    @staticmethod
    def GetGPUAwareMPI():
        return _device.Device_GetGPUAwareMPI()
    GetGPUAwareMPI = _swig_new_static_method(_device.Device_GetGPUAwareMPI)

# Register Device in _device:
_device.Device_swigregister(Device)

def Device_IsConfigured():
    return _device.Device_IsConfigured()
Device_IsConfigured = _device.Device_IsConfigured

def Device_IsAvailable():
    return _device.Device_IsAvailable()
Device_IsAvailable = _device.Device_IsAvailable

def Device_IsEnabled():
    return _device.Device_IsEnabled()
Device_IsEnabled = _device.Device_IsEnabled

def Device_IsDisabled():
    return _device.Device_IsDisabled()
Device_IsDisabled = _device.Device_IsDisabled

def Device_Allows(b_mask):
    return _device.Device_Allows(b_mask)
Device_Allows = _device.Device_Allows

def Device_GetHostMemoryType():
    return _device.Device_GetHostMemoryType()
Device_GetHostMemoryType = _device.Device_GetHostMemoryType

def Device_GetHostMemoryClass():
    return _device.Device_GetHostMemoryClass()
Device_GetHostMemoryClass = _device.Device_GetHostMemoryClass

def Device_GetDeviceMemoryType():
    return _device.Device_GetDeviceMemoryType()
Device_GetDeviceMemoryType = _device.Device_GetDeviceMemoryType

def Device_GetMemoryType():
    return _device.Device_GetMemoryType()
Device_GetMemoryType = _device.Device_GetMemoryType

def Device_GetDeviceMemoryClass():
    return _device.Device_GetDeviceMemoryClass()
Device_GetDeviceMemoryClass = _device.Device_GetDeviceMemoryClass

def Device_GetMemoryClass():
    return _device.Device_GetMemoryClass()
Device_GetMemoryClass = _device.Device_GetMemoryClass

def Device_SetGPUAwareMPI(force=True):
    return _device.Device_SetGPUAwareMPI(force)
Device_SetGPUAwareMPI = _device.Device_SetGPUAwareMPI

def Device_GetGPUAwareMPI():
    return _device.Device_GetGPUAwareMPI()
Device_GetGPUAwareMPI = _device.Device_GetGPUAwareMPI






# This file was automatically generated by SWIG (http://www.swig.org).
# Version 4.0.2
#
# Do not make changes to this file unless you know what you are doing--modify
# the SWIG interface file instead.

from sys import version_info as _swig_python_version_info
if _swig_python_version_info < (2, 7, 0):
    raise RuntimeError("Python 2.7 or later required")

# Import the low-level C/C++ module
if __package__ or "." in __name__:
    from . import _pgridfunc
else:
    import _pgridfunc

try:
    import builtins as __builtin__
except ImportError:
    import __builtin__

_swig_new_instance_method = _pgridfunc.SWIG_PyInstanceMethod_New
_swig_new_static_method = _pgridfunc.SWIG_PyStaticMethod_New

def _swig_repr(self):
    try:
        strthis = "proxy of " + self.this.__repr__()
    except __builtin__.Exception:
        strthis = ""
    return "<%s.%s; %s >" % (self.__class__.__module__, self.__class__.__name__, strthis,)


def _swig_setattr_nondynamic_instance_variable(set):
    def set_instance_attr(self, name, value):
        if name == "thisown":
            self.this.own(value)
        elif name == "this":
            set(self, name, value)
        elif hasattr(self, name) and isinstance(getattr(type(self), name), property):
            set(self, name, value)
        else:
            raise AttributeError("You cannot add instance attributes to %s" % self)
    return set_instance_attr


def _swig_setattr_nondynamic_class_variable(set):
    def set_class_attr(cls, name, value):
        if hasattr(cls, name) and not isinstance(getattr(cls, name), property):
            set(cls, name, value)
        else:
            raise AttributeError("You cannot add class attributes to %s" % cls)
    return set_class_attr


def _swig_add_metaclass(metaclass):
    """Class decorator for adding a metaclass to a SWIG wrapped class - a slimmed down version of six.add_metaclass"""
    def wrapper(cls):
        return metaclass(cls.__name__, cls.__bases__, cls.__dict__.copy())
    return wrapper


class _SwigNonDynamicMeta(type):
    """Meta class to enforce nondynamic attributes (no new attributes) for a class"""
    __setattr__ = _swig_setattr_nondynamic_class_variable(type.__setattr__)


import weakref

MFEM_VERSION = _pgridfunc.MFEM_VERSION

MFEM_VERSION_STRING = _pgridfunc.MFEM_VERSION_STRING

MFEM_VERSION_TYPE = _pgridfunc.MFEM_VERSION_TYPE

MFEM_VERSION_TYPE_RELEASE = _pgridfunc.MFEM_VERSION_TYPE_RELEASE

MFEM_VERSION_TYPE_DEVELOPMENT = _pgridfunc.MFEM_VERSION_TYPE_DEVELOPMENT

MFEM_VERSION_MAJOR = _pgridfunc.MFEM_VERSION_MAJOR

MFEM_VERSION_MINOR = _pgridfunc.MFEM_VERSION_MINOR

MFEM_VERSION_PATCH = _pgridfunc.MFEM_VERSION_PATCH

MFEM_HYPRE_VERSION = _pgridfunc.MFEM_HYPRE_VERSION

import mfem._par.pfespace
import mfem._par.operators
import mfem._par.mem_manager
import mfem._par.vector
import mfem._par.array
import mfem._par.fespace
import mfem._par.coefficient
import mfem._par.globals
import mfem._par.matrix
import mfem._par.intrules
import mfem._par.sparsemat
import mfem._par.densemat
import mfem._par.eltrans
import mfem._par.fe
import mfem._par.geom
import mfem._par.mesh
import mfem._par.sort_pairs
import mfem._par.ncmesh
import mfem._par.vtk
import mfem._par.element
import mfem._par.table
import mfem._par.hash
import mfem._par.vertex
import mfem._par.gridfunc
import mfem._par.bilininteg
import mfem._par.fe_coll
import mfem._par.lininteg
import mfem._par.linearform
import mfem._par.handle
import mfem._par.hypre
import mfem._par.restriction
import mfem._par.pmesh
import mfem._par.pncmesh
import mfem._par.communication
import mfem._par.sets

def GlobalLpNorm(p, loc_norm, comm):
    r"""GlobalLpNorm(double const p, double loc_norm, MPI_Comm comm) -> double"""
    return _pgridfunc.GlobalLpNorm(p, loc_norm, comm)
GlobalLpNorm = _pgridfunc.GlobalLpNorm
class ParGridFunction(mfem._par.gridfunc.GridFunction):
    r"""Proxy of C++ mfem::ParGridFunction class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def ParFESpace(self):
        r"""ParFESpace(ParGridFunction self) -> ParFiniteElementSpace"""
        return _pgridfunc.ParGridFunction_ParFESpace(self)
    ParFESpace = _swig_new_instance_method(_pgridfunc.ParGridFunction_ParFESpace)

    def Update(self):
        r"""Update(ParGridFunction self)"""
        return _pgridfunc.ParGridFunction_Update(self)
    Update = _swig_new_instance_method(_pgridfunc.ParGridFunction_Update)

    def SetSpace(self, *args):
        r"""
        SetSpace(ParGridFunction self, FiniteElementSpace f)
        SetSpace(ParGridFunction self, ParFiniteElementSpace f)
        """
        return _pgridfunc.ParGridFunction_SetSpace(self, *args)
    SetSpace = _swig_new_instance_method(_pgridfunc.ParGridFunction_SetSpace)

    def MakeRef(self, *args):
        r"""
        MakeRef(ParGridFunction self, Vector base, int offset, int size)
        MakeRef(ParGridFunction self, Vector base, int offset)
        MakeRef(ParGridFunction self, FiniteElementSpace f, double * v)
        MakeRef(ParGridFunction self, ParFiniteElementSpace f, double * v)
        MakeRef(ParGridFunction self, FiniteElementSpace f, Vector v, int v_offset)
        MakeRef(ParGridFunction self, ParFiniteElementSpace f, Vector v, int v_offset)
        """
        return _pgridfunc.ParGridFunction_MakeRef(self, *args)
    MakeRef = _swig_new_instance_method(_pgridfunc.ParGridFunction_MakeRef)

    def Distribute(self, *args):
        r"""
        Distribute(ParGridFunction self, Vector tv)
        Distribute(ParGridFunction self, Vector tv)
        """
        return _pgridfunc.ParGridFunction_Distribute(self, *args)
    Distribute = _swig_new_instance_method(_pgridfunc.ParGridFunction_Distribute)

    def AddDistribute(self, *args):
        r"""
        AddDistribute(ParGridFunction self, double a, Vector tv)
        AddDistribute(ParGridFunction self, double a, Vector tv)
        """
        return _pgridfunc.ParGridFunction_AddDistribute(self, *args)
    AddDistribute = _swig_new_instance_method(_pgridfunc.ParGridFunction_AddDistribute)

    def SetFromTrueDofs(self, tv):
        r"""SetFromTrueDofs(ParGridFunction self, Vector tv)"""
        return _pgridfunc.ParGridFunction_SetFromTrueDofs(self, tv)
    SetFromTrueDofs = _swig_new_instance_method(_pgridfunc.ParGridFunction_SetFromTrueDofs)

    def Assign(self, *args):
        r"""
        Assign(ParGridFunction self, ParGridFunction rhs) -> ParGridFunction
        Assign(ParGridFunction self, double value) -> ParGridFunction
        Assign(ParGridFunction self, Vector v) -> ParGridFunction
        Assign(ParGridFunction self, HypreParVector tv) -> ParGridFunction
        """
        return _pgridfunc.ParGridFunction_Assign(self, *args)
    Assign = _swig_new_instance_method(_pgridfunc.ParGridFunction_Assign)

    def GetTrueDofs(self, *args):
        r"""
        GetTrueDofs(ParGridFunction self, Vector tv)
        GetTrueDofs(ParGridFunction self) -> HypreParVector
        """
        return _pgridfunc.ParGridFunction_GetTrueDofs(self, *args)
    GetTrueDofs = _swig_new_instance_method(_pgridfunc.ParGridFunction_GetTrueDofs)

    def ParallelAverage(self, *args):
        r"""
        ParallelAverage(ParGridFunction self, Vector tv)
        ParallelAverage(ParGridFunction self, HypreParVector tv)
        ParallelAverage(ParGridFunction self) -> HypreParVector
        """
        return _pgridfunc.ParGridFunction_ParallelAverage(self, *args)
    ParallelAverage = _swig_new_instance_method(_pgridfunc.ParGridFunction_ParallelAverage)

    def ParallelProject(self, *args):
        r"""
        ParallelProject(ParGridFunction self, Vector tv)
        ParallelProject(ParGridFunction self, HypreParVector tv)
        ParallelProject(ParGridFunction self) -> HypreParVector
        """
        return _pgridfunc.ParGridFunction_ParallelProject(self, *args)
    ParallelProject = _swig_new_instance_method(_pgridfunc.ParGridFunction_ParallelProject)

    def ParallelAssemble(self, *args):
        r"""
        ParallelAssemble(ParGridFunction self, Vector tv)
        ParallelAssemble(ParGridFunction self, HypreParVector tv)
        ParallelAssemble(ParGridFunction self) -> HypreParVector
        """
        return _pgridfunc.ParGridFunction_ParallelAssemble(self, *args)
    ParallelAssemble = _swig_new_instance_method(_pgridfunc.ParGridFunction_ParallelAssemble)

    def ExchangeFaceNbrData(self):
        r"""ExchangeFaceNbrData(ParGridFunction self)"""
        return _pgridfunc.ParGridFunction_ExchangeFaceNbrData(self)
    ExchangeFaceNbrData = _swig_new_instance_method(_pgridfunc.ParGridFunction_ExchangeFaceNbrData)

    def FaceNbrData(self, *args):
        r"""
        FaceNbrData(ParGridFunction self) -> Vector
        FaceNbrData(ParGridFunction self) -> Vector
        """
        return _pgridfunc.ParGridFunction_FaceNbrData(self, *args)
    FaceNbrData = _swig_new_instance_method(_pgridfunc.ParGridFunction_FaceNbrData)

    def GetValue(self, *args):
        r"""
        GetValue(ParGridFunction self, int i, IntegrationPoint ip, int vdim=1) -> double
        GetValue(ParGridFunction self, ElementTransformation T) -> double
        GetValue(ParGridFunction self, ElementTransformation T, IntegrationPoint ip, int comp=0, Vector tr=None) -> double
        """
        return _pgridfunc.ParGridFunction_GetValue(self, *args)
    GetValue = _swig_new_instance_method(_pgridfunc.ParGridFunction_GetValue)

    def GetVectorValue(self, *args):
        r"""
        GetVectorValue(ParGridFunction self, int i, IntegrationPoint ip, Vector val)
        GetVectorValue(ParGridFunction self, ElementTransformation T, IntegrationPoint ip, Vector val, Vector tr=None)
        """
        return _pgridfunc.ParGridFunction_GetVectorValue(self, *args)
    GetVectorValue = _swig_new_instance_method(_pgridfunc.ParGridFunction_GetVectorValue)

    def ProjectCoefficient(self, *args):
        r"""
        ProjectCoefficient(ParGridFunction self, Coefficient coeff)
        ProjectCoefficient(ParGridFunction self, Coefficient coeff, intArray dofs, int vd=0)
        ProjectCoefficient(ParGridFunction self, VectorCoefficient vcoeff)
        ProjectCoefficient(ParGridFunction self, VectorCoefficient vcoeff, intArray dofs)
        ProjectCoefficient(ParGridFunction self, mfem::Coefficient *[] coeff)
        ProjectCoefficient(ParGridFunction self, Coefficient coeff)
        """
        return _pgridfunc.ParGridFunction_ProjectCoefficient(self, *args)
    ProjectCoefficient = _swig_new_instance_method(_pgridfunc.ParGridFunction_ProjectCoefficient)

    def ProjectDiscCoefficient(self, *args):
        r"""
        ProjectDiscCoefficient(ParGridFunction self, VectorCoefficient coeff, intArray dof_attr)
        ProjectDiscCoefficient(ParGridFunction self, VectorCoefficient coeff)
        ProjectDiscCoefficient(ParGridFunction self, Coefficient coeff, mfem::GridFunction::AvgType type)
        ProjectDiscCoefficient(ParGridFunction self, VectorCoefficient coeff, mfem::GridFunction::AvgType type)
        ProjectDiscCoefficient(ParGridFunction self, VectorCoefficient coeff)
        ProjectDiscCoefficient(ParGridFunction self, Coefficient coeff, mfem::GridFunction::AvgType type)
        ProjectDiscCoefficient(ParGridFunction self, VectorCoefficient vcoeff, mfem::GridFunction::AvgType type)
        """
        return _pgridfunc.ParGridFunction_ProjectDiscCoefficient(self, *args)
    ProjectDiscCoefficient = _swig_new_instance_method(_pgridfunc.ParGridFunction_ProjectDiscCoefficient)

    def ProjectBdrCoefficient(self, *args):
        r"""
        ProjectBdrCoefficient(ParGridFunction self, Coefficient coeff, intArray attr)
        ProjectBdrCoefficient(ParGridFunction self, VectorCoefficient vcoeff, intArray attr)
        ProjectBdrCoefficient(ParGridFunction self, mfem::Coefficient *[] coeff, intArray attr)
        ProjectBdrCoefficient(ParGridFunction self, VectorCoefficient vcoeff, intArray attr)
        ProjectBdrCoefficient(ParGridFunction self, mfem::Coefficient *[] coeff, intArray attr)
        """
        return _pgridfunc.ParGridFunction_ProjectBdrCoefficient(self, *args)
    ProjectBdrCoefficient = _swig_new_instance_method(_pgridfunc.ParGridFunction_ProjectBdrCoefficient)

    def ProjectBdrCoefficientTangent(self, vcoeff, bdr_attr):
        r"""ProjectBdrCoefficientTangent(ParGridFunction self, VectorCoefficient vcoeff, intArray bdr_attr)"""
        return _pgridfunc.ParGridFunction_ProjectBdrCoefficientTangent(self, vcoeff, bdr_attr)
    ProjectBdrCoefficientTangent = _swig_new_instance_method(_pgridfunc.ParGridFunction_ProjectBdrCoefficientTangent)

    def ComputeL1Error(self, *args):
        r"""
        ComputeL1Error(ParGridFunction self, mfem::Coefficient *[] exsol, mfem::IntegrationRule const *[] irs=0) -> double
        ComputeL1Error(ParGridFunction self, Coefficient exsol, mfem::IntegrationRule const *[] irs=0) -> double
        ComputeL1Error(ParGridFunction self, VectorCoefficient exsol, mfem::IntegrationRule const *[] irs=0) -> double
        """
        return _pgridfunc.ParGridFunction_ComputeL1Error(self, *args)
    ComputeL1Error = _swig_new_instance_method(_pgridfunc.ParGridFunction_ComputeL1Error)

    def ComputeL2Error(self, *args):
        r"""
        ComputeL2Error(ParGridFunction self, mfem::Coefficient *[] exsol, mfem::IntegrationRule const *[] irs=0) -> double
        ComputeL2Error(ParGridFunction self, Coefficient exsol, mfem::IntegrationRule const *[] irs=0) -> double
        ComputeL2Error(ParGridFunction self, VectorCoefficient exsol, mfem::IntegrationRule const *[] irs=0, intArray elems=None) -> double
        """
        return _pgridfunc.ParGridFunction_ComputeL2Error(self, *args)
    ComputeL2Error = _swig_new_instance_method(_pgridfunc.ParGridFunction_ComputeL2Error)

    def ComputeGradError(self, exgrad, irs=0):
        r"""ComputeGradError(ParGridFunction self, VectorCoefficient exgrad, mfem::IntegrationRule const *[] irs=0) -> double"""
        return _pgridfunc.ParGridFunction_ComputeGradError(self, exgrad, irs)
    ComputeGradError = _swig_new_instance_method(_pgridfunc.ParGridFunction_ComputeGradError)

    def ComputeCurlError(self, excurl, irs=0):
        r"""ComputeCurlError(ParGridFunction self, VectorCoefficient excurl, mfem::IntegrationRule const *[] irs=0) -> double"""
        return _pgridfunc.ParGridFunction_ComputeCurlError(self, excurl, irs)
    ComputeCurlError = _swig_new_instance_method(_pgridfunc.ParGridFunction_ComputeCurlError)

    def ComputeDivError(self, exdiv, irs=0):
        r"""ComputeDivError(ParGridFunction self, Coefficient exdiv, mfem::IntegrationRule const *[] irs=0) -> double"""
        return _pgridfunc.ParGridFunction_ComputeDivError(self, exdiv, irs)
    ComputeDivError = _swig_new_instance_method(_pgridfunc.ParGridFunction_ComputeDivError)

    def ComputeDGFaceJumpError(self, exsol, ell_coeff, Nu, irs=0):
        r"""ComputeDGFaceJumpError(ParGridFunction self, Coefficient exsol, Coefficient ell_coeff, double Nu, mfem::IntegrationRule const *[] irs=0) -> double"""
        return _pgridfunc.ParGridFunction_ComputeDGFaceJumpError(self, exsol, ell_coeff, Nu, irs)
    ComputeDGFaceJumpError = _swig_new_instance_method(_pgridfunc.ParGridFunction_ComputeDGFaceJumpError)

    def ComputeH1Error(self, *args):
        r"""
        ComputeH1Error(ParGridFunction self, Coefficient exsol, VectorCoefficient exgrad, Coefficient ell_coef, double Nu, int norm_type) -> double
        ComputeH1Error(ParGridFunction self, Coefficient exsol, VectorCoefficient exgrad, mfem::IntegrationRule const *[] irs=0) -> double
        """
        return _pgridfunc.ParGridFunction_ComputeH1Error(self, *args)
    ComputeH1Error = _swig_new_instance_method(_pgridfunc.ParGridFunction_ComputeH1Error)

    def ComputeHDivError(self, exsol, exdiv, irs=0):
        r"""ComputeHDivError(ParGridFunction self, VectorCoefficient exsol, Coefficient exdiv, mfem::IntegrationRule const *[] irs=0) -> double"""
        return _pgridfunc.ParGridFunction_ComputeHDivError(self, exsol, exdiv, irs)
    ComputeHDivError = _swig_new_instance_method(_pgridfunc.ParGridFunction_ComputeHDivError)

    def ComputeHCurlError(self, exsol, excurl, irs=0):
        r"""ComputeHCurlError(ParGridFunction self, VectorCoefficient exsol, VectorCoefficient excurl, mfem::IntegrationRule const *[] irs=0) -> double"""
        return _pgridfunc.ParGridFunction_ComputeHCurlError(self, exsol, excurl, irs)
    ComputeHCurlError = _swig_new_instance_method(_pgridfunc.ParGridFunction_ComputeHCurlError)

    def ComputeMaxError(self, *args):
        r"""
        ComputeMaxError(ParGridFunction self, mfem::Coefficient *[] exsol, mfem::IntegrationRule const *[] irs=0) -> double
        ComputeMaxError(ParGridFunction self, Coefficient exsol, mfem::IntegrationRule const *[] irs=0) -> double
        ComputeMaxError(ParGridFunction self, VectorCoefficient exsol, mfem::IntegrationRule const *[] irs=0) -> double
        """
        return _pgridfunc.ParGridFunction_ComputeMaxError(self, *args)
    ComputeMaxError = _swig_new_instance_method(_pgridfunc.ParGridFunction_ComputeMaxError)

    def ComputeLpError(self, *args):
        r"""
        ComputeLpError(ParGridFunction self, double const p, Coefficient exsol, Coefficient weight=None, mfem::IntegrationRule const *[] irs=0) -> double
        ComputeLpError(ParGridFunction self, double const p, VectorCoefficient exsol, Coefficient weight=None, VectorCoefficient v_weight=None, mfem::IntegrationRule const *[] irs=0) -> double
        """
        return _pgridfunc.ParGridFunction_ComputeLpError(self, *args)
    ComputeLpError = _swig_new_instance_method(_pgridfunc.ParGridFunction_ComputeLpError)

    def ComputeFlux(self, blfi, flux, wcoef=True, subdomain=-1):
        r"""ComputeFlux(ParGridFunction self, BilinearFormIntegrator blfi, GridFunction flux, bool wcoef=True, int subdomain=-1)"""
        return _pgridfunc.ParGridFunction_ComputeFlux(self, blfi, flux, wcoef, subdomain)
    ComputeFlux = _swig_new_instance_method(_pgridfunc.ParGridFunction_ComputeFlux)
    __swig_destroy__ = _pgridfunc.delete_ParGridFunction

    def __init__(self, *args):
        r"""
        __init__(ParGridFunction self) -> ParGridFunction
        __init__(ParGridFunction self, ParGridFunction orig) -> ParGridFunction
        __init__(ParGridFunction self, ParFiniteElementSpace pf) -> ParGridFunction
        __init__(ParGridFunction self, ParFiniteElementSpace pf, double * data) -> ParGridFunction
        __init__(ParGridFunction self, ParFiniteElementSpace pf, GridFunction gf) -> ParGridFunction
        __init__(ParGridFunction self, ParFiniteElementSpace pf, HypreParVector tv) -> ParGridFunction
        __init__(ParGridFunction self, ParMesh pmesh, GridFunction gf, int const * partitioning=None) -> ParGridFunction
        __init__(ParGridFunction self, ParMesh pmesh, std::istream & input) -> ParGridFunction
        __init__(ParGridFunction self, ParFiniteElementSpace fes, Vector v, int offset) -> ParGridFunction
        """

        from mfem._par.pmesh import ParMesh
        from mfem._par.pfespace import ParFiniteElementSpace
        from mfem._par.gridfunc import GridFunction
        if (len(args) == 2 and isinstance(args[1], str) and
             isinstance(args[0], ParMesh)):
            g0 = GridFunction(args[0], args[1])
            fec = g0.OwnFEC()
            fes = g0.FESpace()
            pfes = ParFiniteElementSpace(args[0], fec, fes.GetVDim(),
                                              fes.GetOrdering())
            x = ParGridFunction(pfes, g0)
            x.thisown = 0
            pfes.thisown = 0
            g0.thisown = 0
            self.this = x.this
            return 


        _pgridfunc.ParGridFunction_swiginit(self, _pgridfunc.new_ParGridFunction(*args))

    def Save(self, *args):
        r"""
        Save(ParGridFunction self, std::ostream & out)
        Save(ParGridFunction self, char const * file, int precision=8)
        """
        return _pgridfunc.ParGridFunction_Save(self, *args)
    Save = _swig_new_instance_method(_pgridfunc.ParGridFunction_Save)

    def SaveAsOne(self, *args):
        r"""
        SaveAsOne(ParGridFunction self, std::ostream & out=out)
        SaveAsOne(ParGridFunction self, char const * file, int precision=8)
        """
        return _pgridfunc.ParGridFunction_SaveAsOne(self, *args)
    SaveAsOne = _swig_new_instance_method(_pgridfunc.ParGridFunction_SaveAsOne)

# Register ParGridFunction in _pgridfunc:
_pgridfunc.ParGridFunction_swigregister(ParGridFunction)


def L2ZZErrorEstimator(flux_integrator, x, smooth_flux_fes, flux_fes, errors, norm_p=2, solver_tol=1e-12, solver_max_it=200):
    r"""L2ZZErrorEstimator(BilinearFormIntegrator flux_integrator, ParGridFunction x, ParFiniteElementSpace smooth_flux_fes, ParFiniteElementSpace flux_fes, Vector errors, int norm_p=2, double solver_tol=1e-12, int solver_max_it=200) -> double"""
    return _pgridfunc.L2ZZErrorEstimator(flux_integrator, x, smooth_flux_fes, flux_fes, errors, norm_p, solver_tol, solver_max_it)
L2ZZErrorEstimator = _pgridfunc.L2ZZErrorEstimator





# This file was automatically generated by SWIG (http://www.swig.org).
# Version 4.0.2
#
# Do not make changes to this file unless you know what you are doing--modify
# the SWIG interface file instead.

from sys import version_info as _swig_python_version_info
if _swig_python_version_info < (2, 7, 0):
    raise RuntimeError("Python 2.7 or later required")

# Import the low-level C/C++ module
if __package__ or "." in __name__:
    from . import _communication
else:
    import _communication

try:
    import builtins as __builtin__
except ImportError:
    import __builtin__

_swig_new_instance_method = _communication.SWIG_PyInstanceMethod_New
_swig_new_static_method = _communication.SWIG_PyStaticMethod_New

def _swig_repr(self):
    try:
        strthis = "proxy of " + self.this.__repr__()
    except __builtin__.Exception:
        strthis = ""
    return "<%s.%s; %s >" % (self.__class__.__module__, self.__class__.__name__, strthis,)


def _swig_setattr_nondynamic_instance_variable(set):
    def set_instance_attr(self, name, value):
        if name == "thisown":
            self.this.own(value)
        elif name == "this":
            set(self, name, value)
        elif hasattr(self, name) and isinstance(getattr(type(self), name), property):
            set(self, name, value)
        else:
            raise AttributeError("You cannot add instance attributes to %s" % self)
    return set_instance_attr


def _swig_setattr_nondynamic_class_variable(set):
    def set_class_attr(cls, name, value):
        if hasattr(cls, name) and not isinstance(getattr(cls, name), property):
            set(cls, name, value)
        else:
            raise AttributeError("You cannot add class attributes to %s" % cls)
    return set_class_attr


def _swig_add_metaclass(metaclass):
    """Class decorator for adding a metaclass to a SWIG wrapped class - a slimmed down version of six.add_metaclass"""
    def wrapper(cls):
        return metaclass(cls.__name__, cls.__bases__, cls.__dict__.copy())
    return wrapper


class _SwigNonDynamicMeta(type):
    """Meta class to enforce nondynamic attributes (no new attributes) for a class"""
    __setattr__ = _swig_setattr_nondynamic_class_variable(type.__setattr__)


MFEM_VERSION = _communication.MFEM_VERSION

MFEM_VERSION_STRING = _communication.MFEM_VERSION_STRING

MFEM_VERSION_TYPE = _communication.MFEM_VERSION_TYPE

MFEM_VERSION_TYPE_RELEASE = _communication.MFEM_VERSION_TYPE_RELEASE

MFEM_VERSION_TYPE_DEVELOPMENT = _communication.MFEM_VERSION_TYPE_DEVELOPMENT

MFEM_VERSION_MAJOR = _communication.MFEM_VERSION_MAJOR

MFEM_VERSION_MINOR = _communication.MFEM_VERSION_MINOR

MFEM_VERSION_PATCH = _communication.MFEM_VERSION_PATCH

MFEM_HYPRE_VERSION = _communication.MFEM_HYPRE_VERSION

import mfem._par.array
import mfem._par.mem_manager
import mfem._par.table
import mfem._par.sets
class MPI_Session(object):
    r"""Proxy of C++ mfem::MPI_Session class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        r"""
        __init__(MPI_Session self) -> MPI_Session
        __init__(MPI_Session self, int & argc, char **& argv) -> MPI_Session
        """
        _communication.MPI_Session_swiginit(self, _communication.new_MPI_Session(*args))
    __swig_destroy__ = _communication.delete_MPI_Session

    def WorldRank(self):
        r"""WorldRank(MPI_Session self) -> int"""
        return _communication.MPI_Session_WorldRank(self)
    WorldRank = _swig_new_instance_method(_communication.MPI_Session_WorldRank)

    def WorldSize(self):
        r"""WorldSize(MPI_Session self) -> int"""
        return _communication.MPI_Session_WorldSize(self)
    WorldSize = _swig_new_instance_method(_communication.MPI_Session_WorldSize)

    def Root(self):
        r"""Root(MPI_Session self) -> bool"""
        return _communication.MPI_Session_Root(self)
    Root = _swig_new_instance_method(_communication.MPI_Session_Root)

# Register MPI_Session in _communication:
_communication.MPI_Session_swigregister(MPI_Session)

class GroupTopology(object):
    r"""Proxy of C++ mfem::GroupTopology class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        r"""
        __init__(GroupTopology self) -> GroupTopology
        __init__(GroupTopology self, MPI_Comm comm) -> GroupTopology
        __init__(GroupTopology self, GroupTopology gt) -> GroupTopology
        """
        _communication.GroupTopology_swiginit(self, _communication.new_GroupTopology(*args))

    def SetComm(self, comm):
        r"""SetComm(GroupTopology self, MPI_Comm comm)"""
        return _communication.GroupTopology_SetComm(self, comm)
    SetComm = _swig_new_instance_method(_communication.GroupTopology_SetComm)

    def GetComm(self):
        r"""GetComm(GroupTopology self) -> MPI_Comm"""
        return _communication.GroupTopology_GetComm(self)
    GetComm = _swig_new_instance_method(_communication.GroupTopology_GetComm)

    def MyRank(self):
        r"""MyRank(GroupTopology self) -> int"""
        return _communication.GroupTopology_MyRank(self)
    MyRank = _swig_new_instance_method(_communication.GroupTopology_MyRank)

    def NRanks(self):
        r"""NRanks(GroupTopology self) -> int"""
        return _communication.GroupTopology_NRanks(self)
    NRanks = _swig_new_instance_method(_communication.GroupTopology_NRanks)

    def Create(self, groups, mpitag):
        r"""Create(GroupTopology self, ListOfIntegerSets groups, int mpitag)"""
        return _communication.GroupTopology_Create(self, groups, mpitag)
    Create = _swig_new_instance_method(_communication.GroupTopology_Create)

    def NGroups(self):
        r"""NGroups(GroupTopology self) -> int"""
        return _communication.GroupTopology_NGroups(self)
    NGroups = _swig_new_instance_method(_communication.GroupTopology_NGroups)

    def GetNumNeighbors(self):
        r"""GetNumNeighbors(GroupTopology self) -> int"""
        return _communication.GroupTopology_GetNumNeighbors(self)
    GetNumNeighbors = _swig_new_instance_method(_communication.GroupTopology_GetNumNeighbors)

    def GetNeighborRank(self, i):
        r"""GetNeighborRank(GroupTopology self, int i) -> int"""
        return _communication.GroupTopology_GetNeighborRank(self, i)
    GetNeighborRank = _swig_new_instance_method(_communication.GroupTopology_GetNeighborRank)

    def IAmMaster(self, g):
        r"""IAmMaster(GroupTopology self, int g) -> bool"""
        return _communication.GroupTopology_IAmMaster(self, g)
    IAmMaster = _swig_new_instance_method(_communication.GroupTopology_IAmMaster)

    def GetGroupMaster(self, g):
        r"""GetGroupMaster(GroupTopology self, int g) -> int"""
        return _communication.GroupTopology_GetGroupMaster(self, g)
    GetGroupMaster = _swig_new_instance_method(_communication.GroupTopology_GetGroupMaster)

    def GetGroupMasterRank(self, g):
        r"""GetGroupMasterRank(GroupTopology self, int g) -> int"""
        return _communication.GroupTopology_GetGroupMasterRank(self, g)
    GetGroupMasterRank = _swig_new_instance_method(_communication.GroupTopology_GetGroupMasterRank)

    def GetGroupMasterGroup(self, g):
        r"""GetGroupMasterGroup(GroupTopology self, int g) -> int"""
        return _communication.GroupTopology_GetGroupMasterGroup(self, g)
    GetGroupMasterGroup = _swig_new_instance_method(_communication.GroupTopology_GetGroupMasterGroup)

    def GetGroupSize(self, g):
        r"""GetGroupSize(GroupTopology self, int g) -> int"""
        return _communication.GroupTopology_GetGroupSize(self, g)
    GetGroupSize = _swig_new_instance_method(_communication.GroupTopology_GetGroupSize)

    def GetGroup(self, g):
        r"""GetGroup(GroupTopology self, int g) -> int const *"""
        return _communication.GroupTopology_GetGroup(self, g)
    GetGroup = _swig_new_instance_method(_communication.GroupTopology_GetGroup)

    def Load(self, _in):
        r"""Load(GroupTopology self, std::istream & _in)"""
        return _communication.GroupTopology_Load(self, _in)
    Load = _swig_new_instance_method(_communication.GroupTopology_Load)

    def Copy(self, copy):
        r"""Copy(GroupTopology self, GroupTopology copy)"""
        return _communication.GroupTopology_Copy(self, copy)
    Copy = _swig_new_instance_method(_communication.GroupTopology_Copy)
    __swig_destroy__ = _communication.delete_GroupTopology

    def Save(self, *args):
        r"""
        Save(GroupTopology self, std::ostream & out)
        Save(GroupTopology self, char const * file, int precision=8)
        """
        return _communication.GroupTopology_Save(self, *args)
    Save = _swig_new_instance_method(_communication.GroupTopology_Save)

# Register GroupTopology in _communication:
_communication.GroupTopology_swigregister(GroupTopology)

class GroupCommunicator(object):
    r"""Proxy of C++ mfem::GroupCommunicator class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    byGroup = _communication.GroupCommunicator_byGroup
    
    byNeighbor = _communication.GroupCommunicator_byNeighbor
    

    def __init__(self, *args, **kwargs):
        r"""__init__(GroupCommunicator self, GroupTopology gt, mfem::GroupCommunicator::Mode m=byNeighbor) -> GroupCommunicator"""
        _communication.GroupCommunicator_swiginit(self, _communication.new_GroupCommunicator(*args, **kwargs))

    def Create(self, ldof_group):
        r"""Create(GroupCommunicator self, intArray ldof_group)"""
        return _communication.GroupCommunicator_Create(self, ldof_group)
    Create = _swig_new_instance_method(_communication.GroupCommunicator_Create)

    def GroupLDofTable(self, *args):
        r"""
        GroupLDofTable(GroupCommunicator self) -> Table
        GroupLDofTable(GroupCommunicator self) -> Table
        """
        return _communication.GroupCommunicator_GroupLDofTable(self, *args)
    GroupLDofTable = _swig_new_instance_method(_communication.GroupCommunicator_GroupLDofTable)

    def Finalize(self):
        r"""Finalize(GroupCommunicator self)"""
        return _communication.GroupCommunicator_Finalize(self)
    Finalize = _swig_new_instance_method(_communication.GroupCommunicator_Finalize)

    def SetLTDofTable(self, ldof_ltdof):
        r"""SetLTDofTable(GroupCommunicator self, intArray ldof_ltdof)"""
        return _communication.GroupCommunicator_SetLTDofTable(self, ldof_ltdof)
    SetLTDofTable = _swig_new_instance_method(_communication.GroupCommunicator_SetLTDofTable)

    def GetGroupTopology(self, *args):
        r"""
        GetGroupTopology(GroupCommunicator self) -> GroupTopology
        GetGroupTopology(GroupCommunicator self) -> GroupTopology
        """
        return _communication.GroupCommunicator_GetGroupTopology(self, *args)
    GetGroupTopology = _swig_new_instance_method(_communication.GroupCommunicator_GetGroupTopology)

    def GetNeighborLTDofTable(self, nbr_ltdof):
        r"""GetNeighborLTDofTable(GroupCommunicator self, Table nbr_ltdof)"""
        return _communication.GroupCommunicator_GetNeighborLTDofTable(self, nbr_ltdof)
    GetNeighborLTDofTable = _swig_new_instance_method(_communication.GroupCommunicator_GetNeighborLTDofTable)

    def GetNeighborLDofTable(self, nbr_ldof):
        r"""GetNeighborLDofTable(GroupCommunicator self, Table nbr_ldof)"""
        return _communication.GroupCommunicator_GetNeighborLDofTable(self, nbr_ldof)
    GetNeighborLDofTable = _swig_new_instance_method(_communication.GroupCommunicator_GetNeighborLDofTable)
    __swig_destroy__ = _communication.delete_GroupCommunicator

    def PrintInfo(self, *args):
        r"""
        PrintInfo(GroupCommunicator self, std::ostream & out=mfem::out)
        PrintInfo(GroupCommunicator self, char const * file, int precision=8)
        """
        return _communication.GroupCommunicator_PrintInfo(self, *args)
    PrintInfo = _swig_new_instance_method(_communication.GroupCommunicator_PrintInfo)

# Register GroupCommunicator in _communication:
_communication.GroupCommunicator_swigregister(GroupCommunicator)


def ReorderRanksZCurve(comm):
    r"""ReorderRanksZCurve(MPI_Comm comm) -> MPI_Comm"""
    return _communication.ReorderRanksZCurve(comm)
ReorderRanksZCurve = _communication.ReorderRanksZCurve


